# Generated by Django 4.2.28 on 2026-02-26 07:21

# Django
import django.core.validators
import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    replaces = [
        ("taxsystem", "0001_initial"),
        ("taxsystem", "0002_alter_paymenthistory_comment_and_more"),
        ("taxsystem", "0003_alter_general_options_alter_paymenthistory_comment"),
        ("taxsystem", "0004_owneraudit_last_update_payday_and_more"),
        (
            "taxsystem",
            "0005_rename_division_corporationwalletdivision_division_id_and_more",
        ),
        ("taxsystem", "0006_create_update_status_existings"),
        ("taxsystem", "0007_alter_ownerupdatestatus_section"),
        (
            "taxsystem",
            "0008_journalfilter_journalfilterset_delete_filteramount_and_more",
        ),
        ("taxsystem", "0009_payments_corporation_id"),
        ("taxsystem", "0010_create_alliance_corporation"),
        ("taxsystem", "0011_alter_corporationowner_options_and_more"),
        ("taxsystem", "0012_alliancefilter_alliancefilterset_and_more"),
        ("taxsystem", "0013_rename_adminlogs_corporationadminhistory_and_more"),
        (
            "taxsystem",
            "0014_rename_allianceownerupdatestatus_allianceupdatestatus_and_more",
        ),
        ("taxsystem", "0015_alter_general_options"),
        ("taxsystem", "0016_remove_corporationpayments_corporation_id_and_more"),
        ("taxsystem", "0017_alter_allianceupdatestatus_unique_together_and_more"),
        ("taxsystem", "0018_alter_allianceowner_corporation"),
        ("taxsystem", "0019_alter_alliancepayments_entry_id_and_more"),
        ("taxsystem", "0020_alter_general_options_and_more"),
        (
            "taxsystem",
            "0021_remove_alliancepayments_taxsystem_a_account_677604_idx_and_more",
        ),
        ("taxsystem", "0022_alter_alliancepaymenthistory_new_status_and_more"),
        ("taxsystem", "0023_alliancefilter_match_type_and_more"),
        ("taxsystem", "0024_alter_allianceadminhistory_action_and_more"),
        ("taxsystem", "0025_eveentity_and_more"),
        (
            "taxsystem",
            "0026_remove_corporationwalletjournalentry_taxsystem_c_first_p_d83db8_idx_and_more",
        ),
        (
            "taxsystem",
            "0027_rename_first_party_new_corporationwalletjournalentry_first_party_and_more",
        ),
    ]

    initial = True

    dependencies = [
        ("eveonline", "0017_alliance_and_corp_names_are_not_unique"),
        ("contenttypes", "0002_remove_content_type_name"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="OwnerAudit",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("active", models.BooleanField(default=True)),
                (
                    "tax_amount",
                    models.DecimalField(
                        decimal_places=0,
                        default=0,
                        help_text="Tax Amount in ISK that is set for the corporation. Max 16 Digits",
                        max_digits=16,
                        validators=[
                            django.core.validators.MaxValueValidator(9999999999999999)
                        ],
                    ),
                ),
                (
                    "tax_period",
                    models.PositiveIntegerField(
                        default=30,
                        help_text="Tax Period in days for the corporation. Max 365 days. Default: 30 days",
                        validators=[django.core.validators.MaxValueValidator(365)],
                    ),
                ),
                (
                    "alliance",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="alliance",
                        to="eveonline.eveallianceinfo",
                    ),
                ),
                (
                    "corporation",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="eveonline.evecorporationinfo",
                    ),
                ),
            ],
            options={
                "verbose_name": "Tax System Audit",
                "verbose_name_plural": "Tax System Audits",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="EveEntity",
            fields=[
                ("id", models.BigIntegerField(primary_key=True, serialize=False)),
                ("name", models.CharField(max_length=255)),
                ("category", models.CharField(max_length=50)),
                ("last_updated", models.DateTimeField(auto_now=True)),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="PaymentSystem",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("date", models.DateTimeField(auto_now_add=True, null=True)),
                (
                    "status",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("active", "Active"),
                            ("inactive", "Inactive"),
                            ("deactivated", "Deactivated"),
                            ("missing", "Missing"),
                        ],
                        default="active",
                        max_length=16,
                    ),
                ),
                (
                    "deposit",
                    models.DecimalField(
                        decimal_places=0,
                        default=0,
                        help_text="Deposit Pool in ISK. Max 16 Digits",
                        max_digits=16,
                        validators=[
                            django.core.validators.MaxValueValidator(9999999999999999),
                            django.core.validators.MinValueValidator(-9999999999999999),
                        ],
                    ),
                ),
                ("last_paid", models.DateTimeField(blank=True, null=True)),
                ("notice", models.TextField(blank=True, null=True)),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_payment_system",
                        to="taxsystem.owneraudit",
                    ),
                ),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Tax Payment System",
                "verbose_name_plural": "Tax Payment Systems",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="Payments",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("entry_id", models.BigIntegerField(blank=True, null=True)),
                ("amount", models.DecimalField(decimal_places=0, max_digits=12)),
                ("date", models.DateTimeField(blank=True, null=True)),
                ("reason", models.TextField(blank=True, null=True)),
                (
                    "request_status",
                    models.CharField(
                        choices=[
                            ("approved", "Approved"),
                            ("pending", "Pending"),
                            ("rejected", "Rejected"),
                            ("needs_approval", "Requires Auditor"),
                        ],
                        default="pending",
                        max_length=16,
                        verbose_name="Request Status",
                    ),
                ),
                (
                    "reviser",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="Reviser that approved or rejected the payment",
                        max_length=100,
                    ),
                ),
                (
                    "account",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_payments",
                        to="taxsystem.paymentsystem",
                    ),
                ),
            ],
            options={
                "verbose_name": "Tax Payments",
                "verbose_name_plural": "Tax Payments",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="CorporationWalletDivision",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(default=None, max_length=100, null=True)),
                ("balance", models.DecimalField(decimal_places=2, max_digits=20)),
                ("division_id", models.IntegerField()),
                (
                    "corporation",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_corporation_division",
                        to="taxsystem.owneraudit",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="General",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "permissions": (
                    ("basic_access", "Can access the Tax System"),
                    ("create_access", "Can add Corporation"),
                    ("manage_own_corp", "Can manage own Corporation"),
                    ("manage_corps", "Can manage all Corporations"),
                ),
                "managed": False,
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="PaymentHistory",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "date",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="Date of the action",
                        verbose_name="Date",
                    ),
                ),
                (
                    "action",
                    models.CharField(
                        choices=[
                            ("", ""),
                            ("Status Changed", "Status Changed"),
                            ("Payment Added", "Payment Added"),
                            ("Reviser Comment", "Reviser Comment"),
                        ],
                        default="",
                        help_text="Action performed",
                        max_length=20,
                        verbose_name="Action",
                    ),
                ),
                (
                    "comment",
                    models.CharField(
                        choices=[
                            ("", ""),
                            ("Payment added to system", "Payment added to system"),
                            (
                                "Automated approved Payment",
                                "Automated approved Payment",
                            ),
                            (
                                "Payment must be approved by an reviser",
                                "Payment must be approved by an reviser",
                            ),
                        ],
                        default="",
                        help_text="Comment of the action",
                        max_length=255,
                        verbose_name="Comment",
                    ),
                ),
                (
                    "new_status",
                    models.CharField(
                        choices=[
                            ("approved", "Approved"),
                            ("pending", "Pending"),
                            ("rejected", "Rejected"),
                            ("needs_approval", "Requires Auditor"),
                        ],
                        help_text="New Status of the action",
                        max_length=16,
                        verbose_name="New Status",
                    ),
                ),
                (
                    "payment",
                    models.ForeignKey(
                        help_text="Payment that the action was performed on",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="taxsystem.payments",
                        verbose_name="Payment",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        help_text="User that performed the action",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="User",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="Members",
            fields=[
                ("character_name", models.CharField(db_index=True, max_length=100)),
                (
                    "character_id",
                    models.PositiveIntegerField(primary_key=True, serialize=False),
                ),
                (
                    "status",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("active", "Active"),
                            ("missing", "Missing"),
                            ("noaccount", "Unregistered"),
                            ("is_alt", "Is Alt"),
                        ],
                        default="active",
                        max_length=10,
                        verbose_name="Status",
                    ),
                ),
                ("logon", models.DateTimeField(blank=True, null=True)),
                ("logged_off", models.DateTimeField(blank=True, null=True)),
                ("joined", models.DateTimeField(blank=True, null=True)),
                ("notice", models.TextField(blank=True, null=True)),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_members",
                        to="taxsystem.owneraudit",
                    ),
                ),
            ],
            options={
                "verbose_name": "Tax Member System",
                "verbose_name_plural": "Tax Member Systems",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AdminLogs",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "date",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="Date of the action",
                        verbose_name="Date",
                    ),
                ),
                (
                    "action",
                    models.CharField(
                        choices=[
                            ("", ""),
                            ("Added", "Added"),
                            ("Changed", "Changed"),
                            ("Deleted", "Deleted"),
                        ],
                        default="",
                        help_text="Action performed",
                        max_length=20,
                        verbose_name="Action",
                    ),
                ),
                (
                    "comment",
                    models.TextField(blank=True, default="", verbose_name="Comment"),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        help_text="Payment that the action was performed on",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="taxsystem.owneraudit",
                        verbose_name="Payment",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        help_text="User that performed the action",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="User",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="CorporationWalletJournalEntry",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "amount",
                    models.DecimalField(
                        decimal_places=2, default=None, max_digits=20, null=True
                    ),
                ),
                (
                    "balance",
                    models.DecimalField(
                        decimal_places=2, default=None, max_digits=20, null=True
                    ),
                ),
                ("context_id", models.BigIntegerField(default=None, null=True)),
                (
                    "context_id_type",
                    models.CharField(
                        choices=[
                            ("structure_id", "Structure Id"),
                            ("station_id", "Station Id"),
                            ("market_transaction_id", "Market Transaction Id"),
                            ("character_id", "Character Id"),
                            ("corporation_id", "Corporation Id"),
                            ("alliance_id", "Alliance Id"),
                            ("eve_system", "Eve System"),
                            ("industry_job_id", "Industry Job Id"),
                            ("contract_id", "Contract Id"),
                            ("planet_id", "Planet Id"),
                            ("system_id", "System Id"),
                            ("type_id", "Type Id"),
                        ],
                        default=None,
                        max_length=30,
                        null=True,
                    ),
                ),
                ("date", models.DateTimeField()),
                ("description", models.CharField(max_length=500)),
                ("entry_id", models.BigIntegerField()),
                ("reason", models.CharField(default=None, max_length=500, null=True)),
                ("ref_type", models.CharField(max_length=72)),
                (
                    "tax",
                    models.DecimalField(
                        decimal_places=2, default=None, max_digits=20, null=True
                    ),
                ),
                ("tax_receiver_id", models.IntegerField(default=None, null=True)),
                (
                    "division",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_corporation_wallet",
                        to="taxsystem.corporationwalletdivision",
                    ),
                ),
                (
                    "first_party",
                    models.ForeignKey(
                        blank=True,
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.SET_DEFAULT,
                        related_name="+",
                        to="taxsystem.eveentity",
                    ),
                ),
                (
                    "second_party",
                    models.ForeignKey(
                        blank=True,
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.SET_DEFAULT,
                        related_name="+",
                        to="taxsystem.eveentity",
                    ),
                ),
            ],
            options={
                "abstract": False,
                "default_permissions": (),
                "indexes": [
                    models.Index(fields=["date"], name="taxsystem_c_date_1a3f46_idx"),
                    models.Index(
                        fields=["amount"], name="taxsystem_c_amount_3f4bd8_idx"
                    ),
                    models.Index(
                        fields=["entry_id"], name="taxsystem_c_entry_i_bd9090_idx"
                    ),
                    models.Index(
                        fields=["ref_type"], name="taxsystem_c_ref_typ_5f82e6_idx"
                    ),
                    models.Index(
                        fields=["first_party"], name="taxsystem_c_first_p_d83db8_idx"
                    ),
                    models.Index(
                        fields=["second_party"], name="taxsystem_c_second__2a759e_idx"
                    ),
                ],
            },
        ),
        migrations.CreateModel(
            name="OwnerUpdateStatus",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "section",
                    models.CharField(
                        choices=[
                            ("wallet", "Wallet Journal"),
                            ("division_names", "Wallet Division Names"),
                            ("division", "Wallet Division"),
                            ("members", "Members"),
                            ("payments", "Payments"),
                            ("payment_system", "Payment System"),
                            ("payday", "Payday"),
                        ],
                        db_index=True,
                        max_length=32,
                    ),
                ),
                (
                    "is_success",
                    models.BooleanField(db_index=True, default=None, null=True),
                ),
                ("error_message", models.TextField()),
                ("has_token_error", models.BooleanField(default=False)),
                (
                    "last_run_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last run has been started at this time",
                        null=True,
                    ),
                ),
                (
                    "last_run_finished_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last run has been successful finished at this time",
                        null=True,
                    ),
                ),
                (
                    "last_update_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last update has been started at this time",
                        null=True,
                    ),
                ),
                (
                    "last_update_finished_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last update has been successful finished at this time",
                        null=True,
                    ),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_update_status",
                        to="taxsystem.owneraudit",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="JournalFilterSet",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100, unique=True)),
                ("description", models.CharField(blank=True, max_length=255)),
                ("enabled", models.BooleanField(default=True)),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_filter_set",
                        to="taxsystem.owneraudit",
                    ),
                ),
            ],
            options={
                "verbose_name": "Journal Filter Set",
                "verbose_name_plural": "Journal Filter Sets",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="JournalFilter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "filter_type",
                    models.CharField(
                        choices=[("reason", "Reason"), ("amount", "Amount")],
                        max_length=20,
                    ),
                ),
                ("value", models.CharField(max_length=255)),
                (
                    "filter_set",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_filters",
                        to="taxsystem.journalfilterset",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.AddConstraint(
            model_name="journalfilter",
            constraint=models.UniqueConstraint(
                fields=("filter_set", "value"), name="unique_filter_value_per_set"
            ),
        ),
        migrations.AddField(
            model_name="payments",
            name="corporation_id",
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.RenameModel(
            old_name="OwnerAudit",
            new_name="CorporationOwner",
        ),
        migrations.RenameModel(
            old_name="Payments",
            new_name="CorporationPayments",
        ),
        migrations.RenameModel(
            old_name="PaymentSystem",
            new_name="CorporationPaymentAccount",
        ),
        migrations.RenameModel(
            old_name="OwnerUpdateStatus",
            new_name="CorporationUpdateStatus",
        ),
        migrations.CreateModel(
            name="AllianceOwner",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("active", models.BooleanField(default=True)),
                (
                    "tax_amount",
                    models.DecimalField(
                        decimal_places=0,
                        default=0,
                        help_text="Tax Amount in ISK that is set for the corporation. Max 16 Digits",
                        max_digits=16,
                        validators=[
                            django.core.validators.MaxValueValidator(9999999999999999)
                        ],
                    ),
                ),
                (
                    "tax_period",
                    models.PositiveIntegerField(
                        default=30,
                        help_text="Tax Period in days for the corporation. Max 365 days. Default: 30 days",
                        validators=[django.core.validators.MaxValueValidator(365)],
                    ),
                ),
                (
                    "eve_alliance",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="eveonline.eveallianceinfo",
                    ),
                ),
                (
                    "corporation",
                    models.ForeignKey(
                        help_text="The corporation that owns this alliance tax system.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="taxsystem.corporationowner",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AllianceOwnerUpdateStatus",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "is_success",
                    models.BooleanField(db_index=True, default=None, null=True),
                ),
                ("error_message", models.TextField()),
                ("has_token_error", models.BooleanField(default=False)),
                (
                    "last_run_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last run has been started at this time",
                        null=True,
                    ),
                ),
                (
                    "last_run_finished_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last run has been successful finished at this time",
                        null=True,
                    ),
                ),
                (
                    "last_update_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last update has been started at this time",
                        null=True,
                    ),
                ),
                (
                    "last_update_finished_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last update has been successful finished at this time",
                        null=True,
                    ),
                ),
                (
                    "section",
                    models.CharField(
                        choices=[
                            ("payments", "Payments"),
                            ("payment_system", "Payment System"),
                            ("payday", "Payday"),
                        ],
                        db_index=True,
                        max_length=32,
                    ),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_alliance_update_status",
                        to="taxsystem.allianceowner",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AlliancePaymentAccount",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("date", models.DateTimeField(auto_now_add=True, null=True)),
                (
                    "status",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("active", "Active"),
                            ("inactive", "Inactive"),
                            ("deactivated", "Deactivated"),
                            ("missing", "Missing"),
                        ],
                        default="active",
                        max_length=16,
                    ),
                ),
                (
                    "deposit",
                    models.DecimalField(
                        decimal_places=0,
                        default=0,
                        help_text="Deposit Pool in ISK. Max 16 Digits",
                        max_digits=16,
                        validators=[
                            django.core.validators.MaxValueValidator(9999999999999999),
                            django.core.validators.MinValueValidator(-9999999999999999),
                        ],
                    ),
                ),
                ("last_paid", models.DateTimeField(blank=True, null=True)),
                ("notice", models.TextField(blank=True, null=True)),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_alliance_payment_accounts",
                        to="taxsystem.allianceowner",
                    ),
                ),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AlliancePayments",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("entry_id", models.BigIntegerField(blank=True, null=True)),
                ("amount", models.DecimalField(decimal_places=0, max_digits=12)),
                ("date", models.DateTimeField(blank=True, null=True)),
                ("reason", models.TextField(blank=True, null=True)),
                (
                    "request_status",
                    models.CharField(
                        choices=[
                            ("approved", "Approved"),
                            ("pending", "Pending"),
                            ("rejected", "Rejected"),
                            ("needs_approval", "Requires Auditor"),
                        ],
                        default="pending",
                        max_length=16,
                        verbose_name="Request Status",
                    ),
                ),
                (
                    "reviser",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="Reviser that approved or rejected the payment",
                        max_length=100,
                    ),
                ),
                (
                    "account",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_alliance_payments",
                        to="taxsystem.alliancepaymentaccount",
                    ),
                ),
                (
                    "owner_id",
                    models.PositiveIntegerField(
                        blank=True,
                        help_text="ID of the owner (corporation or alliance) associated with this payment",
                        null=True,
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.AlterModelOptions(
            name="corporationowner",
            options={"default_permissions": ()},
        ),
        migrations.AlterModelOptions(
            name="corporationpaymentaccount",
            options={"default_permissions": ()},
        ),
        migrations.AlterModelOptions(
            name="corporationpayments",
            options={"default_permissions": ()},
        ),
        migrations.RenameField(
            model_name="corporationowner",
            old_name="corporation",
            new_name="eve_corporation",
        ),
        migrations.RemoveField(
            model_name="corporationowner",
            name="alliance",
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corp_payments",
                to="taxsystem.corporationpaymentaccount",
            ),
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_update_status",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="journalfilter",
            name="filter_set",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_filters",
                to="taxsystem.journalfilterset",
            ),
        ),
        migrations.AlterField(
            model_name="journalfilterset",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_filter_set",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.CreateModel(
            name="AllianceFilter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "filter_type",
                    models.CharField(
                        choices=[("reason", "Reason"), ("amount", "Amount")],
                        max_length=20,
                    ),
                ),
                ("value", models.CharField(max_length=255)),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AllianceFilterSet",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100, unique=True)),
                ("description", models.CharField(blank=True, max_length=255)),
                ("enabled", models.BooleanField(default=True)),
            ],
            options={
                "abstract": False,
                "default_permissions": (),
            },
        ),
        migrations.RenameModel(
            old_name="JournalFilter",
            new_name="CorporationFilter",
        ),
        migrations.RenameModel(
            old_name="JournalFilterSet",
            new_name="CorporationFilterSet",
        ),
        migrations.AlterModelOptions(
            name="corporationfilterset",
            options={"default_permissions": ()},
        ),
        migrations.AlterModelOptions(
            name="members",
            options={"default_permissions": ()},
        ),
        migrations.RemoveConstraint(
            model_name="corporationfilter",
            name="unique_filter_value_per_set",
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="section",
            field=models.CharField(choices=[], db_index=True, max_length=32),
        ),
        migrations.AddField(
            model_name="alliancefilterset",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_alliance_filter_set",
                to="taxsystem.allianceowner",
            ),
        ),
        migrations.AddField(
            model_name="alliancefilter",
            name="filter_set",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_alliance_filters",
                to="taxsystem.alliancefilterset",
            ),
        ),
        migrations.RenameModel(
            old_name="AdminLogs",
            new_name="CorporationAdminHistory",
        ),
        migrations.RenameModel(
            old_name="PaymentHistory",
            new_name="CorporationPaymentHistory",
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_payment_accounts",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_payments",
                to="taxsystem.corporationpaymentaccount",
            ),
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("wallet", "Wallet Journal"),
                    ("division_names", "Wallet Division Names"),
                    ("division", "Wallet Division"),
                    ("members", "Members"),
                    ("payments", "Payments"),
                    ("payment_system", "Payment System"),
                    ("payday", "Payday"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
        migrations.RenameModel(
            old_name="AllianceOwnerUpdateStatus",
            new_name="AllianceUpdateStatus",
        ),
        migrations.CreateModel(
            name="AlliancePaymentHistory",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "date",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="Date of the action",
                        verbose_name="Date",
                    ),
                ),
                (
                    "action",
                    models.CharField(
                        choices=[
                            ("", ""),
                            ("Status Changed", "Status Changed"),
                            ("Payment Added", "Payment Added"),
                            ("Reviser Comment", "Reviser Comment"),
                        ],
                        default="",
                        help_text="Action performed",
                        max_length=20,
                        verbose_name="Action",
                    ),
                ),
                (
                    "comment",
                    models.CharField(
                        choices=[
                            ("", ""),
                            ("Payment added to system", "Payment added to system"),
                            (
                                "Automated approved Payment",
                                "Automated approved Payment",
                            ),
                            (
                                "Payment must be approved by an reviser",
                                "Payment must be approved by an reviser",
                            ),
                        ],
                        default="",
                        help_text="Comment of the action",
                        max_length=255,
                        verbose_name="Comment",
                    ),
                ),
                (
                    "new_status",
                    models.CharField(
                        choices=[
                            ("approved", "Approved"),
                            ("pending", "Pending"),
                            ("rejected", "Rejected"),
                            ("needs_approval", "Requires Auditor"),
                        ],
                        help_text="New Status of the action",
                        max_length=16,
                        verbose_name="New Status",
                    ),
                ),
                (
                    "payment",
                    models.ForeignKey(
                        help_text="Payment that the action was performed on",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="taxsystem.alliancepayments",
                        verbose_name="Payment",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        help_text="User that performed the action",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="User",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AllianceAdminHistory",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "date",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="Date of the action",
                        verbose_name="Date",
                    ),
                ),
                (
                    "action",
                    models.CharField(
                        choices=[
                            ("", ""),
                            ("Added", "Added"),
                            ("Changed", "Changed"),
                            ("Deleted", "Deleted"),
                        ],
                        default="",
                        help_text="Action performed",
                        max_length=20,
                        verbose_name="Action",
                    ),
                ),
                (
                    "comment",
                    models.TextField(blank=True, default="", verbose_name="Comment"),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        help_text="Payment that the action was performed on",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="taxsystem.allianceowner",
                        verbose_name="Payment",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        help_text="User that performed the action",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="User",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access the Tax System"),
                    ("create_access", "Can add Corporation"),
                    ("manage_own_corp", "Can manage own Corporation"),
                    ("manage_corps", "Can manage all Corporations"),
                    ("manage_own_alliance", "Can manage own Alliance"),
                    ("manage_alliances", "Can manage all Alliances"),
                ),
            },
        ),
        migrations.AlterField(
            model_name="allianceupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("payment_system", "Payment System"),
                    ("payments", "Payments"),
                    ("payday", "Payday"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("division_names", "Wallet Division Names"),
                    ("division", "Wallet Division"),
                    ("wallet", "Wallet Journal"),
                    ("members", "Members"),
                    ("payment_system", "Payment System"),
                    ("payments", "Payments"),
                    ("payday", "Payday"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
        migrations.RemoveField(
            model_name="corporationpayments",
            name="corporation_id",
        ),
        migrations.AddField(
            model_name="corporationpayments",
            name="owner_id",
            field=models.PositiveIntegerField(
                blank=True,
                help_text="ID of the owner (corporation or alliance) associated with this payment",
                null=True,
            ),
        ),
        migrations.AddIndex(
            model_name="alliancepayments",
            index=models.Index(
                fields=["account", "owner_id", "request_status", "-date"],
                name="taxsystem_a_account_677604_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="alliancepayments",
            index=models.Index(
                fields=["request_status", "-date"],
                name="taxsystem_a_request_d243c8_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="corporationpayments",
            index=models.Index(
                fields=["account", "owner_id", "request_status", "-date"],
                name="taxsystem_c_account_dafabb_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="corporationpayments",
            index=models.Index(
                fields=["request_status", "-date"],
                name="taxsystem_c_request_5fb377_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="members",
            index=models.Index(
                fields=["owner", "character_name"],
                name="taxsystem_m_owner_i_f6738f_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="members",
            index=models.Index(fields=["status"], name="taxsystem_m_status_38e108_idx"),
        ),
        migrations.AlterUniqueTogether(
            name="allianceupdatestatus",
            unique_together={("owner", "section")},
        ),
        migrations.AlterUniqueTogether(
            name="corporationupdatestatus",
            unique_together={("owner", "section")},
        ),
        migrations.AlterField(
            model_name="allianceowner",
            name="corporation",
            field=models.ForeignKey(
                help_text="The corporation that owns this alliance tax system.",
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access the Tax System"),
                    ("create_access", "Can add Corporation/Alliance"),
                    ("manage_own_corp", "Can manage own Corporation"),
                    ("manage_corps", "Can manage all Corporations"),
                    ("manage_own_alliance", "Can manage own Alliance"),
                    ("manage_alliances", "Can manage all Alliances"),
                ),
            },
        ),
        migrations.AlterField(
            model_name="allianceadminhistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="allianceadminhistory",
            name="owner",
            field=models.ForeignKey(
                help_text="Owner that the action was performed on",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_admin_history",
                to="taxsystem.allianceowner",
                verbose_name="Owner",
            ),
        ),
        migrations.AlterField(
            model_name="alliancefilter",
            name="value",
            field=models.CharField(max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name="allianceowner",
            name="active",
            field=models.BooleanField(
                default=True,
                help_text="Designates whether this alliance owner is active.",
            ),
        ),
        migrations.AlterField(
            model_name="allianceowner",
            name="tax_amount",
            field=models.DecimalField(
                decimal_places=0,
                default=0,
                help_text="Tax Amount in ISK that is set for the alliance. Max 16 Digits",
                max_digits=16,
                validators=[django.core.validators.MaxValueValidator(9999999999999999)],
            ),
        ),
        migrations.AlterField(
            model_name="allianceowner",
            name="tax_period",
            field=models.PositiveIntegerField(
                default=30,
                help_text="Tax Period in days for the alliance. Max 365 days. Default: 30 days",
                validators=[django.core.validators.MaxValueValidator(365)],
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymentaccount",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_ally_tax_accounts",
                to="taxsystem.allianceowner",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Status Changed", "Status Changed"),
                    ("Payment Added", "Payment Added"),
                    ("Custom Payment Added", "Custom Payment Added"),
                    ("Reviser Comment", "Reviser Comment"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationadminhistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="corporationadminhistory",
            name="owner",
            field=models.ForeignKey(
                help_text="Owner that the action was performed on",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_admin_history",
                to="taxsystem.corporationowner",
                verbose_name="Owner",
            ),
        ),
        migrations.AlterField(
            model_name="corporationfilter",
            name="value",
            field=models.CharField(max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name="corporationowner",
            name="active",
            field=models.BooleanField(
                default=True,
                help_text="Designates whether this corporation owner is active.",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corp_tax_accounts",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Status Changed", "Status Changed"),
                    ("Payment Added", "Payment Added"),
                    ("Custom Payment Added", "Custom Payment Added"),
                    ("Reviser Comment", "Reviser Comment"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.RemoveIndex(
            model_name="alliancepayments",
            name="taxsystem_a_account_677604_idx",
        ),
        migrations.RemoveIndex(
            model_name="alliancepayments",
            name="taxsystem_a_request_d243c8_idx",
        ),
        migrations.RemoveIndex(
            model_name="corporationpayments",
            name="taxsystem_c_account_dafabb_idx",
        ),
        migrations.RemoveIndex(
            model_name="corporationpayments",
            name="taxsystem_c_request_5fb377_idx",
        ),
        migrations.RemoveIndex(
            model_name="members",
            name="taxsystem_m_owner_i_f6738f_idx",
        ),
        migrations.RemoveIndex(
            model_name="members",
            name="taxsystem_m_status_38e108_idx",
        ),
        migrations.RemoveField(
            model_name="alliancepayments",
            name="owner_id",
        ),
        migrations.RemoveField(
            model_name="corporationpayments",
            name="owner_id",
        ),
        migrations.AddField(
            model_name="alliancepayments",
            name="journal",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.corporationwalletjournalentry",
            ),
        ),
        migrations.AddField(
            model_name="alliancepayments",
            name="owner",
            field=models.ForeignKey(
                blank=True,
                help_text="Owner of this payment",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_alliance_payments",
                to="taxsystem.allianceowner",
            ),
        ),
        migrations.AddField(
            model_name="corporationpayments",
            name="journal",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.corporationwalletjournalentry",
            ),
        ),
        migrations.AddField(
            model_name="corporationpayments",
            name="owner",
            field=models.ForeignKey(
                blank=True,
                help_text="Owner of this payment",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_payments",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepayments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.alliancepaymentaccount",
            ),
        ),
        migrations.AlterField(
            model_name="allianceupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("tax_accounts", "Tax Accounts"),
                    ("payments", "Payments"),
                    ("deadlines", "Deadlines"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.corporationpaymentaccount",
            ),
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("division_names", "Wallet Division Names"),
                    ("divisions", "Wallet Divisions"),
                    ("wallet", "Wallet Journal"),
                    ("members", "Members"),
                    ("tax_accounts", "Tax Accounts"),
                    ("payments", "Payments"),
                    ("deadlines", "Deadlines"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "approved"),
                    ("pending", "pending"),
                    ("rejected", "rejected"),
                    ("needs_approval", "requires auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepayments",
            name="request_status",
            field=models.CharField(
                choices=[
                    ("approved", "approved"),
                    ("pending", "pending"),
                    ("rejected", "rejected"),
                    ("needs_approval", "requires auditor"),
                ],
                default="pending",
                max_length=16,
                verbose_name="Request Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "approved"),
                    ("pending", "pending"),
                    ("rejected", "rejected"),
                    ("needs_approval", "requires auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="request_status",
            field=models.CharField(
                choices=[
                    ("approved", "approved"),
                    ("pending", "pending"),
                    ("rejected", "rejected"),
                    ("needs_approval", "requires auditor"),
                ],
                default="pending",
                max_length=16,
                verbose_name="Request Status",
            ),
        ),
        migrations.AlterField(
            model_name="allianceadminhistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Added", "added"),
                    ("Changed", "changed"),
                    ("Deleted", "deleted"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="corporationadminhistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Added", "added"),
                    ("Changed", "changed"),
                    ("Deleted", "deleted"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymentaccount",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "active"),
                    ("inactive", "inactive"),
                    ("deactivated", "deactivated"),
                    ("missing", "missing"),
                ],
                default="active",
                max_length=16,
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "active"),
                    ("inactive", "inactive"),
                    ("deactivated", "deactivated"),
                    ("missing", "missing"),
                ],
                default="active",
                max_length=16,
            ),
        ),
        migrations.AlterField(
            model_name="members",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "active"),
                    ("missing", "missing"),
                    ("noaccount", "unregistered"),
                    ("is_alt", "is alt"),
                ],
                default="active",
                max_length=10,
                verbose_name="Status",
            ),
        ),
        migrations.AddField(
            model_name="alliancefilter",
            name="match_type",
            field=models.CharField(
                choices=[("exact", "Exact Match"), ("contains", "Contains")],
                default="exact",
                max_length=20,
            ),
        ),
        migrations.AddField(
            model_name="corporationfilter",
            name="match_type",
            field=models.CharField(
                choices=[("exact", "Exact Match"), ("contains", "Contains")],
                default="exact",
                max_length=20,
            ),
        ),
        migrations.AlterField(
            model_name="allianceadminhistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Added", "Added"),
                    ("Changed", "Changed"),
                    ("Deleted", "Deleted"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymentaccount",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "Active"),
                    ("inactive", "Inactive"),
                    ("deactivated", "Deactivated"),
                    ("missing", "Missing"),
                ],
                default="active",
                max_length=16,
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepayments",
            name="request_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                default="pending",
                max_length=16,
                verbose_name="Request Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationadminhistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Added", "Added"),
                    ("Changed", "Changed"),
                    ("Deleted", "Deleted"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "Active"),
                    ("inactive", "Inactive"),
                    ("deactivated", "Deactivated"),
                    ("missing", "Missing"),
                ],
                default="active",
                max_length=16,
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="request_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                default="pending",
                max_length=16,
                verbose_name="Request Status",
            ),
        ),
        migrations.AlterField(
            model_name="members",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "Active"),
                    ("missing", "Missing"),
                    ("noaccount", "Unregistered"),
                    ("is_alt", "is alt"),
                ],
                default="active",
                max_length=10,
                verbose_name="Status",
            ),
        ),
    ]



# Django
from django.test import TestCase
from django.utils import timezone

# Alliance Auth
from allianceauth.tests.auth_utils import AuthUtils

# AA TaxSystem
from taxsystem.models.alliance import AlliancePayments
from taxsystem.models.corporation import CorporationPayments
from taxsystem.models.general import EveEntity
from taxsystem.models.helpers.textchoices import (
    PaymentRequestStatus,
)
from taxsystem.tests import TaxSystemTestCase
from taxsystem.tests.testdata.utils import (
    create_division,
    create_owner_from_user,
    create_payment,
    create_tax_account,
    create_wallet_journal_entry,
)

MODULE_PATH = "taxsystem.models.tax"


class TestPaymentsModel(TaxSystemTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.audit = create_owner_from_user(cls.user)
        cls.audit2 = create_owner_from_user(cls.superuser)

        cls.eve_character_first_party = EveEntity.objects.get(id=2001)
        cls.eve_character_second_party = EveEntity.objects.get(id=1001)

        cls.division = create_division(
            corporation=cls.audit,
            division_id=1,
            name="Main Division",
            balance=1000000,
        )

        cls.journal_entry = create_wallet_journal_entry(
            division=cls.division,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Test Journal Entry",
            ref_type="tax_payment",
            first_party=cls.eve_character_first_party,
            second_party=cls.eve_character_second_party,
            description="Test Description",
        )

        cls.tax_account = create_tax_account(
            name=cls.user_character.character.character_name,
            owner=cls.audit,
            user=cls.user,
            deposit=0,
        )

        cls.payments = create_payment(
            name="Gneuten",
            amount=1000,
            request_status="needs_approval",
            account=cls.tax_account,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reviser="Requires Auditor",
            journal=cls.journal_entry,
            owner=cls.audit,
        )

    def test_str(self):
        expected_str = CorporationPayments.objects.get(account=self.tax_account)
        self.assertEqual(self.payments, expected_str)

    def test_is_automatic(self):
        """Test if the payment is automatic."""
        payments = CorporationPayments.objects.get(account=self.tax_account)
        self.assertFalse(payments.is_automatic)

    def test_is_pending(self):
        """Test if the payment is pending."""
        self.payments.request_status = PaymentRequestStatus.PENDING
        self.payments.save()

        payments = CorporationPayments.objects.get(account=self.tax_account)
        self.assertTrue(payments.is_pending)

    def test_is_approved(self):
        """Test if the payment is approved."""
        self.payments.request_status = PaymentRequestStatus.APPROVED
        self.payments.save()

        payments = CorporationPayments.objects.get(account=self.tax_account)
        self.assertFalse(payments.is_pending)
        self.assertTrue(payments.is_approved)

    def test_is_rejected(self):
        """Test if the payment is rejected."""
        self.payments.request_status = PaymentRequestStatus.REJECTED
        self.payments.save()

        payments = CorporationPayments.objects.get(account=self.tax_account)
        self.assertFalse(payments.is_pending)
        self.assertTrue(payments.is_rejected)

    def test_character_id(self):
        """Test if the character_id is correct."""
        payments = CorporationPayments.objects.get(account=self.tax_account)
        self.assertEqual(
            payments.character_id, self.user_character.character.character_id
        )

    def test_division(self):
        """Test if the division is correct."""
        payments = CorporationPayments.objects.get(account=self.tax_account)
        self.assertEqual(payments.division_name, "Main Division")

    def test_access_no_perms(self):
        """Test should return only own corporation if basic_access is set."""
        corporation = CorporationPayments.objects.get_visible(user=self.user2)
        self.assertNotIn(self.payments, corporation)

    def test_access_perms_own_corp(self):
        """Test should return only own corporation if manage_own_corp is set."""
        self.user = AuthUtils.add_permission_to_user_by_name(
            "taxsystem.manage_own_corp", self.user
        )
        self.user.refresh_from_db()
        corporation = CorporationPayments.objects.get_visible(user=self.user)
        self.assertIn(self.payments, corporation)

    def test_access_perms_manage_corps(self):
        """Test should return all corporations if manage_corps is set."""
        self.user = AuthUtils.add_permission_to_user_by_name(
            "taxsystem.manage_corps", self.user
        )
        self.user.refresh_from_db()
        corporation = CorporationPayments.objects.get_visible(user=self.user)
        self.assertIn(self.payments, corporation)

    def test_open_invoices_count(self):
        """Test the count of open invoices."""
        self.payments.request_status = PaymentRequestStatus.PENDING
        self.payments.save()

        open_invoices = CorporationPayments.objects.get_owner_open_invoices(
            user=self.superuser, owner=self.audit
        )
        self.assertEqual(open_invoices, 1)

    def test_open_invoices_count_is_none(self):
        """Test the count of open invoices."""
        self.payments.request_status = PaymentRequestStatus.PENDING
        self.payments.save()

        open_invoices = CorporationPayments.objects.get_owner_open_invoices(
            user=self.user, owner=self.audit
        )
        self.assertEqual(open_invoices, 0)

    def test_get_visible_invoices(self):
        """Test the count of open invoices."""
        self.payments.request_status = PaymentRequestStatus.PENDING
        self.payments.save()

        open_invoices = CorporationPayments.objects.get_visible_open_invoices(
            user=self.manage_own_user
        )
        self.assertEqual(open_invoices, 1)

    def test_get_visible_invoices_is_none(self):
        """Test the count of open invoices."""
        self.payments.request_status = PaymentRequestStatus.PENDING
        self.payments.save()

        open_invoices = CorporationPayments.objects.get_visible_open_invoices(
            user=self.user2
        )
        self.assertEqual(open_invoices, 0)


class TestAlliancePaymentsModel(TaxSystemTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.corp_audit = create_owner_from_user(user=cls.user)
        cls.audit = create_owner_from_user(user=cls.user, tax_type="alliance")
        cls.audit2 = create_owner_from_user(user=cls.superuser, tax_type="alliance")

        cls.eve_character_first_party = EveEntity.objects.get(id=2001)
        cls.eve_character_second_party = EveEntity.objects.get(id=1001)

        cls.division = create_division(
            corporation=cls.corp_audit,
            division_id=1,
            name="Main Division",
            balance=1000000,
        )

        cls.journal_entry = create_wallet_journal_entry(
            division=cls.division,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Test Journal Entry",
            ref_type="tax_payment",
            first_party=cls.eve_character_first_party,
            second_party=cls.eve_character_second_party,
            description="Test Description",
        )

        cls.tax_account = create_tax_account(
            name=cls.user_character.character.character_name,
            owner=cls.audit,
            user=cls.user,
            deposit=0,
        )

        cls.payments = create_payment(
            name="Gneuten",
            amount=1000,
            request_status="needs_approval",
            account=cls.tax_account,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reviser="Requires Auditor",
            journal=cls.journal_entry,
            owner=cls.audit,
        )

    def test_str(self):
        expected_str = AlliancePayments.objects.get(account=self.tax_account)
        self.assertEqual(self.payments, expected_str)

    def test_is_automatic(self):
        """Test if the payment is automatic."""
        payments = AlliancePayments.objects.get(account=self.tax_account)
        self.assertFalse(payments.is_automatic)

    def test_is_pending(self):
        """Test if the payment is pending."""
        self.payments.request_status = PaymentRequestStatus.PENDING
        self.payments.save()

        payments = AlliancePayments.objects.get(account=self.tax_account)
        self.assertTrue(payments.is_pending)

    def test_is_approved(self):
        """Test if the payment is approved."""
        self.payments.request_status = PaymentRequestStatus.APPROVED
        self.payments.save()

        payments = AlliancePayments.objects.get(account=self.tax_account)
        self.assertFalse(payments.is_pending)
        self.assertTrue(payments.is_approved)

    def test_is_rejected(self):
        """Test if the payment is rejected."""
        self.payments.request_status = PaymentRequestStatus.REJECTED
        self.payments.save()

        payments = AlliancePayments.objects.get(account=self.tax_account)
        self.assertFalse(payments.is_pending)
        self.assertTrue(payments.is_rejected)

    def test_character_id(self):
        """Test if the character_id is correct."""
        payments = AlliancePayments.objects.get(account=self.tax_account)
        self.assertEqual(
            payments.character_id, self.user_character.character.character_id
        )

    def test_division(self):
        """Test if the division is correct."""
        payments = AlliancePayments.objects.get(account=self.tax_account)
        self.assertEqual(payments.division_name, "Main Division")

    def test_access_no_perms(self):
        """Test should return only own corporation if basic_access is set."""
        corporation = AlliancePayments.objects.get_visible(user=self.user2)
        self.assertNotIn(self.payments, corporation)

    def test_access_perms_own_alliance(self):
        """Test should return only own alliance if manage_own_alliance is set."""
        self.user = AuthUtils.add_permission_to_user_by_name(
            "taxsystem.manage_own_alliance", self.user
        )
        self.user.refresh_from_db()
        corporation = AlliancePayments.objects.get_visible(user=self.user)
        self.assertIn(self.payments, corporation)

    def test_access_perms_manage_alliances(self):
        """Test should return all alliances if manage_alliances is set."""
        self.user = AuthUtils.add_permission_to_user_by_name(
            "taxsystem.manage_alliances", self.user
        )
        self.user.refresh_from_db()
        corporation = AlliancePayments.objects.get_visible(user=self.user)
        self.assertIn(self.payments, corporation)

    def test_open_invoices_count(self):
        """Test the count of open invoices."""
        self.payments.request_status = PaymentRequestStatus.PENDING
        self.payments.save()

        open_invoices = AlliancePayments.objects.get_owner_open_invoices(
            user=self.superuser, owner=self.audit
        )
        self.assertEqual(open_invoices, 1)

    def test_open_invoices_count_is_none(self):
        """Test the count of open invoices."""
        self.payments.request_status = PaymentRequestStatus.PENDING
        self.payments.save()

        open_invoices = AlliancePayments.objects.get_owner_open_invoices(
            user=self.user, owner=self.audit
        )
        self.assertEqual(open_invoices, 0)

    def test_get_visible_invoices(self):
        """Test the count of open invoices."""
        self.payments.request_status = PaymentRequestStatus.PENDING
        self.payments.save()

        open_invoices = AlliancePayments.objects.get_visible_open_invoices(
            user=self.manage_own_user
        )
        self.assertEqual(open_invoices, 1)

    def test_get_visible_invoices_is_none(self):
        """Test the count of open invoices."""
        self.payments.request_status = PaymentRequestStatus.PENDING
        self.payments.save()

        open_invoices = AlliancePayments.objects.get_visible_open_invoices(
            user=self.user2
        )
        self.assertEqual(open_invoices, 0)



"""Models for Tax System."""

# Standard Library
from typing import TYPE_CHECKING

# Django
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.utils import timezone
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.authentication.models import User
from allianceauth.services.hooks import get_extension_logger

# AA TaxSystem
from taxsystem import __title__, app_settings
from taxsystem.models.helpers.textchoices import (
    AccountStatus,
    FilterMatchType,
    PaymentActions,
    PaymentRequestStatus,
    PaymentStatus,
)
from taxsystem.providers import AppLogger

logger = AppLogger(get_extension_logger(__name__), __title__)

if TYPE_CHECKING:
    # AA TaxSystem
    from taxsystem.models.wallet import CorporationWalletJournalEntry


class PaymentsBaseModel(models.Model):
    """Basemodel for Payments in Tax System"""

    class Meta:
        abstract = True
        default_permissions = ()

    name = models.CharField(max_length=100)

    entry_id = models.BigIntegerField(null=True, blank=True)

    journal = models.OneToOneField(
        "CorporationWalletJournalEntry",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        related_name="+",
    )

    amount = models.DecimalField(max_digits=12, decimal_places=0)

    date = models.DateTimeField(null=True, blank=True)

    reason = models.TextField(null=True, blank=True)

    request_status = models.CharField(
        max_length=16,
        choices=PaymentRequestStatus.choices,
        default=PaymentRequestStatus.PENDING,
        verbose_name=_("Request Status"),
    )

    reviser = models.CharField(
        max_length=100,
        blank=True,
        default="",
        help_text=_("Reviser that approved or rejected the payment"),
    )

    @property
    def is_automatic(self) -> bool:
        return self.reviser == "System"

    @property
    def is_pending(self) -> bool:
        return self.request_status == PaymentRequestStatus.PENDING

    @property
    def is_needs_approval(self) -> bool:
        return self.request_status == PaymentRequestStatus.NEEDS_APPROVAL

    @property
    def is_approved(self) -> bool:
        return self.request_status == PaymentRequestStatus.APPROVED

    @property
    def is_rejected(self) -> bool:
        return self.request_status == PaymentRequestStatus.REJECTED

    @property
    def character_id(self) -> int:
        """
        Returns Main Character ID or Journal Character ID associated with this payment.

        Returns:
            int: Character ID
        """
        try:
            character_id = self.account.user.profile.main_character.character_id
            return character_id
        except AttributeError:
            return self.journal.first_party_id

    @property
    def division_name(self) -> "CorporationWalletJournalEntry":
        """
        Returns:
            str: The name of the division or "N/A".
        """
        if not self.journal:
            return "N/A"
        return self.journal.division.name

    def get_request_status(self) -> str:
        return self.get_request_status_display()

    @property
    def formatted_payment_date(self) -> str:
        if self.date:
            return timezone.localtime(self.date).strftime("%Y-%m-%d %H:%M:%S")
        return _("No date")


class PaymentAccountBaseModel(models.Model):
    """Basemodel for Payment User Accounts in Tax System"""

    class Meta:
        abstract = True
        default_permissions = ()

    name = models.CharField(
        max_length=100,
    )

    user = models.OneToOneField(User, on_delete=models.CASCADE, related_name="+")

    date = models.DateTimeField(auto_now_add=True, null=True, blank=True)

    status = models.CharField(
        max_length=16,
        choices=AccountStatus.choices,
        blank=True,
        default=AccountStatus.ACTIVE,
    )

    deposit = models.DecimalField(
        max_digits=16,
        decimal_places=0,
        default=0,
        help_text=_("Deposit Pool in ISK. Max 16 Digits"),
        validators=[
            MaxValueValidator(9999999999999999),
            MinValueValidator(-9999999999999999),
        ],
    )

    last_paid = models.DateTimeField(null=True, blank=True)

    notice = models.TextField(null=True, blank=True)

    def __str__(self):
        return f"{self.name} - {self.date} - {self.status}"

    def get_payment_status(self) -> str:
        return self.get_status_display()

    def get_alt_ids(self) -> list[int]:
        return list(
            self.user.character_ownerships.all().values_list(
                "character__character_id", flat=True
            )
        )

    @property
    def is_active(self) -> bool:
        return self.status == AccountStatus.ACTIVE

    @property
    def is_inactive(self) -> bool:
        return self.status == AccountStatus.INACTIVE

    @property
    def is_deactivated(self) -> bool:
        return self.status == AccountStatus.DEACTIVATED

    @property
    def is_missing(self) -> bool:
        return self.status == AccountStatus.MISSING

    @property
    def has_paid(self) -> bool:
        """
        Return True if user has paid for alliance.

        Returns:
            bool: True if paid, False otherwise.
        """
        subclass = getattr(self, "owner", None)
        if not subclass:
            raise NotImplementedError(
                "has_paid property must be implemented in subclass"
            )

        if self.deposit >= self.owner.tax_amount:
            return True
        if self.last_paid and self.deposit >= 0:
            return (timezone.now() - self.last_paid) < timezone.timedelta(
                days=self.owner.tax_period
            )
        return False

    @property
    def next_due(self):
        """
        Return the next due date for alliance payment.

        Returns:
            datetime or None: Next due date or None if inactive/deactivated or never paid.
        """
        subclass = getattr(self, "owner", None)
        if not subclass:
            raise NotImplementedError(
                "has_paid property must be implemented in subclass"
            )

        if self.status in [AccountStatus.INACTIVE, AccountStatus.DEACTIVATED]:
            return None
        if self.last_paid:
            return self.last_paid + timezone.timedelta(days=self.owner.tax_period)
        return None

    def has_paid_icon(self, badge=False, text=False) -> str:
        """
        Return the HTML icon for has_paid.

        Returns:
            str: HTML icon string.
        """
        color = "success" if self.has_paid else "danger"

        if self.has_paid:
            html = f"<i class='fas fa-check' title='{PaymentStatus('paid').label}' data-bs-tooltip='aa-taxsystem'></i>"
        else:
            html = f"<i class='fas fa-times' title='{PaymentStatus('unpaid').label}' data-bs-tooltip='aa-taxsystem'></i>"

        if text:
            html += f" {PaymentStatus('paid').label if self.has_paid else PaymentStatus('unpaid').label}"

        if badge:
            html = mark_safe(f"<span class='badge bg-{color}'>{html}</span>")
        return html


class UpdateStatusBaseModel(models.Model):
    """Base Model for owner update status."""

    class Meta:
        abstract = True
        default_permissions = ()

    is_success = models.BooleanField(default=None, null=True, db_index=True)
    error_message = models.TextField()
    has_token_error = models.BooleanField(default=False)

    last_run_at = models.DateTimeField(
        default=None,
        null=True,
        db_index=True,
        help_text="Last run has been started at this time",
    )
    last_run_finished_at = models.DateTimeField(
        default=None,
        null=True,
        db_index=True,
        help_text="Last run has been successful finished at this time",
    )
    last_update_at = models.DateTimeField(
        default=None,
        null=True,
        db_index=True,
        help_text="Last update has been started at this time",
    )
    last_update_finished_at = models.DateTimeField(
        default=None,
        null=True,
        db_index=True,
        help_text="Last update has been successful finished at this time",
    )

    def need_update(self) -> bool:
        """Check if the update is needed."""
        if not self.is_success or not self.last_update_finished_at:
            needs_update = True
        else:
            section_time_stale = app_settings.TAXSYSTEM_STALE_TYPES.get(
                self.section, 60
            )
            stale = timezone.now() - timezone.timedelta(minutes=section_time_stale)
            needs_update = self.last_run_finished_at <= stale

        if needs_update and self.has_token_error:
            logger.info(
                "%s: Ignoring update because of token error, section: %s",
                self.owner,
                self.section,
            )
            needs_update = False

        return needs_update

    def reset(self) -> None:
        """Reset this update status."""
        self.is_success = None
        self.error_message = ""
        self.has_token_error = False
        self.last_run_at = timezone.now()
        self.last_run_finished_at = None
        self.save()


class FilterBaseModel(models.Model):
    class Meta:
        abstract = True
        default_permissions = ()

    class FilterType(models.TextChoices):
        REASON = "reason", _("Reason")
        AMOUNT = "amount", _("Amount")

    filter_type = models.CharField(max_length=20, choices=FilterType.choices)
    match_type = models.CharField(
        max_length=20,
        choices=FilterMatchType.choices,
        default=FilterMatchType.EXACT,
    )
    value = models.CharField(max_length=255, unique=True)

    def get_match_type_filter(self) -> models.Q:
        """
        Generate a Q object based on the filter type and match type.

        Returns:
            models.Q: The generated Q object for filtering.
        """
        if self.match_type == FilterMatchType.CONTAINS:
            return models.Q(**{f"{self.filter_type}__icontains": self.value})
        return models.Q(**{self.filter_type: self.value})

    def __str__(self) -> str:
        return f"Filter: {self.filter_type}({self.match_type}) = {self.value}"


class FilterSetBaseModel(models.Model):
    class Meta:
        abstract = True
        default_permissions = ()

    name = models.CharField(max_length=100, unique=True)
    description = models.CharField(max_length=255, blank=True)
    enabled = models.BooleanField(default=True)

    def __str__(self):
        return self.name

    @property
    def is_active(self) -> bool:
        return self.enabled

    @property
    def is_active_html(self) -> mark_safe:
        if self.enabled:
            return mark_safe('<i class="fa-solid fa-check"></i>')
        return mark_safe('<i class="fa-solid fa-times"></i>')

    def filter(
        self, payments: models.QuerySet  # pylint: disable=unused-argument
    ) -> models.QuerySet:
        raise NotImplementedError("Create filter method")


class HistoryBaseModel(models.Model):
    """Basemodel for Payments History"""

    class Meta:
        abstract = True
        default_permissions = ()

    user = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        related_name="+",
        verbose_name=_("User"),
        help_text=_("User that performed the action"),
    )

    date = models.DateTimeField(
        default=timezone.now,
        verbose_name=_("Date"),
        help_text=_("Date of the action"),
    )

    action = models.CharField(
        max_length=20,
        choices=PaymentActions.choices,
        default=PaymentActions.DEFAULT,
        verbose_name=_("Action"),
        help_text=_("Action performed"),
    )

    comment = models.TextField(
        blank=True,
        default="",
        verbose_name=_("Comment"),
        help_text=_("Comment of the action"),
    )

    def __str__(self):
        return f"{self.date}: {self.user} - {self.action} - {self.comment}"



# Standard Library
from typing import TYPE_CHECKING

# Django
from django.db import models
from django.db.models import Case, Count, Q, Value, When

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# AA TaxSystem
from taxsystem import __title__
from taxsystem.models.helpers.textchoices import (
    AllianceUpdateSection,
    CorporationUpdateSection,
    UpdateStatus,
)
from taxsystem.providers import AppLogger

logger = AppLogger(get_extension_logger(__name__), __title__)

if TYPE_CHECKING:
    # AA TaxSystem
    from taxsystem.models.alliance import AllianceOwner
    from taxsystem.models.corporation import CorporationOwner


class CorporationOwnerQuerySet(models.QuerySet["CorporationOwner"]):
    """QuerySet for CorporationOwner with common filtering logic."""

    def visible_to(self, user):
        """Get all corps visible to the user."""
        # superusers get all visible
        if user.is_superuser:
            logger.debug(
                "Returning all corps for superuser %s.",
                user,
            )
            return self

        if user.has_perm("taxsystem.manage_corps"):
            logger.debug("Returning all corps for Tax Audit Manager %s.", user)
            return self

        try:
            char = user.profile.main_character
            assert char
            corp_ids = user.character_ownerships.all().values_list(
                "character__corporation_id", flat=True
            )
            queries = [models.Q(eve_corporation__corporation_id__in=corp_ids)]

            logger.debug(
                "%s queries for user %s visible corporations.", len(queries), user
            )

            query = queries.pop()
            for q in queries:
                query |= q
            return self.filter(query)
        except AssertionError:
            logger.debug("User %s has no main character. Nothing visible.", user)
            return self.none()

    def manage_to(self, user):
        """Get all corps that the user can manage."""
        # superusers get all visible
        if user.is_superuser:
            logger.debug(
                "Returning all corps for superuser %s.",
                user,
            )
            return self

        if user.has_perm("taxsystem.manage_corps"):
            logger.debug("Returning all corps for Tax Audit Manager %s.", user)
            return self

        try:
            char = user.profile.main_character
            assert char
            query = None

            if user.has_perm("taxsystem.manage_own_corp"):
                query = models.Q(eve_corporation__corporation_id=char.corporation_id)

            logger.debug("Returning own corps for User %s.", user)

            if query is None:
                return self.none()

            return self.filter(query)
        except AssertionError:
            logger.debug("User %s has no main character. Nothing visible.", user)
            return self.none()

    def annotate_total_update_status_user(self, user):
        """Get the total update status for the given user."""
        char = user.profile.main_character
        assert char

        query = models.Q(character__character_ownership__user=user)

        return self.filter(query).annotate_total_update_status()

    def annotate_total_update_status(self):
        """Get the total update status."""
        sections = CorporationUpdateSection.get_sections()
        num_sections_total = len(sections)
        qs = (
            self.annotate(
                num_sections_total=Count(
                    "ts_corporation_update_status",
                    filter=Q(ts_corporation_update_status__section__in=sections),
                )
            )
            .annotate(
                num_sections_ok=Count(
                    "ts_corporation_update_status",
                    filter=Q(
                        ts_corporation_update_status__section__in=sections,
                        ts_corporation_update_status__is_success=True,
                    ),
                )
            )
            .annotate(
                num_sections_failed=Count(
                    "ts_corporation_update_status",
                    filter=Q(
                        ts_corporation_update_status__section__in=sections,
                        ts_corporation_update_status__is_success=False,
                    ),
                )
            )
            .annotate(
                num_sections_token_error=Count(
                    "ts_corporation_update_status",
                    filter=Q(
                        ts_corporation_update_status__section__in=sections,
                        ts_corporation_update_status__has_token_error=True,
                    ),
                )
            )
            # pylint: disable=no-member
            .annotate(
                total_update_status=Case(
                    When(
                        active=False,
                        then=Value(UpdateStatus.DISABLED),
                    ),
                    When(
                        num_sections_token_error=1,
                        then=Value(UpdateStatus.TOKEN_ERROR),
                    ),
                    When(
                        num_sections_failed__gt=0,
                        then=Value(UpdateStatus.ERROR),
                    ),
                    When(
                        num_sections_ok=num_sections_total,
                        then=Value(UpdateStatus.OK),
                    ),
                    When(
                        num_sections_total__lt=num_sections_total,
                        then=Value(UpdateStatus.INCOMPLETE),
                    ),
                    default=Value(UpdateStatus.IN_PROGRESS),
                )
            )
        )

        return qs


class CorporationOwnerManager(models.Manager["CorporationOwner"]):
    def get_queryset(self):
        return CorporationOwnerQuerySet(self.model, using=self._db)

    def visible_to(self, user):
        return self.get_queryset().visible_to(user)

    def manage_to(self, user):
        return self.get_queryset().manage_to(user)

    def annotate_total_update_status(self):
        return self.get_queryset().annotate_total_update_status()


class AllianceOwnerQuerySet(models.QuerySet["AllianceOwner"]):
    """QuerySet for AllianceOwner with common filtering logic."""

    def visible_to(self, user):
        """Get all allys visible to the user."""
        # superusers get all visible
        if user.is_superuser:
            logger.debug(
                "Returning all alliances for superuser %s.",
                user,
            )
            return self

        if user.has_perm("taxsystem.manage_alliances"):
            logger.debug("Returning all alliances for Tax Audit Manager %s.", user)
            return self

        try:
            char = user.profile.main_character
            assert char
            alliance_ids = user.character_ownerships.all().values_list(
                "character__alliance_id", flat=True
            )
            queries = [models.Q(eve_alliance__alliance_id__in=alliance_ids)]

            logger.debug(
                "%s queries for user %s visible alliances.", len(queries), user
            )

            query = queries.pop()
            for q in queries:
                query |= q
            return self.filter(query)
        except AssertionError:
            logger.debug("User %s has no main character. Nothing visible.", user)
            return self.none()

    def manage_to(self, user):
        """Get all alliances that the user can manage."""
        # superusers get all visible
        if user.is_superuser:
            logger.debug(
                "Returning all alliances for superuser %s.",
                user,
            )
            return self

        if user.has_perm("taxsystem.manage_alliances"):
            logger.debug("Returning all alliances for Tax Audit Manager %s.", user)
            return self

        try:
            char = user.profile.main_character
            assert char
            query = None

            if user.has_perm("taxsystem.manage_own_alliance"):
                query = models.Q(eve_alliance__alliance_id=char.alliance_id)

            logger.debug("Returning own alliances for User %s.", user)

            if query is None:
                return self.none()

            return self.filter(query)
        except AssertionError:
            logger.debug("User %s has no main character. Nothing visible.", user)
            return self.none()

    # pylint: disable=duplicate-code
    def annotate_total_update_status(self):
        """Get the total update status."""
        sections = AllianceUpdateSection.get_sections()
        num_sections_total = len(sections)
        qs = (
            self.annotate(
                num_sections_total=Count(
                    "ts_alliance_update_status",
                    filter=Q(ts_alliance_update_status__section__in=sections),
                )
            )
            .annotate(
                num_sections_ok=Count(
                    "ts_alliance_update_status",
                    filter=Q(
                        ts_alliance_update_status__section__in=sections,
                        ts_alliance_update_status__is_success=True,
                    ),
                )
            )
            .annotate(
                num_sections_failed=Count(
                    "ts_alliance_update_status",
                    filter=Q(
                        ts_alliance_update_status__section__in=sections,
                        ts_alliance_update_status__is_success=False,
                    ),
                )
            )
            .annotate(
                num_sections_token_error=Count(
                    "ts_alliance_update_status",
                    filter=Q(
                        ts_alliance_update_status__section__in=sections,
                        ts_alliance_update_status__has_token_error=True,
                    ),
                )
            )
            # pylint: disable=no-member
            .annotate(
                total_update_status=Case(
                    When(
                        active=False,
                        then=Value(UpdateStatus.DISABLED),
                    ),
                    When(
                        num_sections_token_error=1,
                        then=Value(UpdateStatus.TOKEN_ERROR),
                    ),
                    When(
                        num_sections_failed__gt=0,
                        then=Value(UpdateStatus.ERROR),
                    ),
                    When(
                        num_sections_ok=num_sections_total,
                        then=Value(UpdateStatus.OK),
                    ),
                    When(
                        num_sections_total__lt=num_sections_total,
                        then=Value(UpdateStatus.INCOMPLETE),
                    ),
                    default=Value(UpdateStatus.IN_PROGRESS),
                )
            )
        )

        return qs


class AllianceOwnerManager(models.Manager["AllianceOwner"]):
    def get_queryset(self):
        return AllianceOwnerQuerySet(self.model, using=self._db)

    def visible_to(self, user):
        return self.get_queryset().visible_to(user)

    def manage_to(self, user):
        return self.get_queryset().manage_to(user)

    def annotate_total_update_status(self):
        return self.get_queryset().annotate_total_update_status()



# Generated by Django 4.2.26 on 2025-12-20 00:45

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0020_alter_general_options_and_more"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="alliancepayments",
            name="taxsystem_a_account_677604_idx",
        ),
        migrations.RemoveIndex(
            model_name="alliancepayments",
            name="taxsystem_a_request_d243c8_idx",
        ),
        migrations.RemoveIndex(
            model_name="corporationpayments",
            name="taxsystem_c_account_dafabb_idx",
        ),
        migrations.RemoveIndex(
            model_name="corporationpayments",
            name="taxsystem_c_request_5fb377_idx",
        ),
        migrations.RemoveIndex(
            model_name="members",
            name="taxsystem_m_owner_i_f6738f_idx",
        ),
        migrations.RemoveIndex(
            model_name="members",
            name="taxsystem_m_status_38e108_idx",
        ),
        migrations.RemoveField(
            model_name="alliancepayments",
            name="owner_id",
        ),
        migrations.RemoveField(
            model_name="corporationpayments",
            name="owner_id",
        ),
        migrations.AddField(
            model_name="alliancepayments",
            name="journal",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.corporationwalletjournalentry",
            ),
        ),
        migrations.AddField(
            model_name="alliancepayments",
            name="owner",
            field=models.ForeignKey(
                blank=True,
                help_text="Owner of this payment",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_alliance_payments",
                to="taxsystem.allianceowner",
            ),
        ),
        migrations.AddField(
            model_name="corporationpayments",
            name="journal",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.corporationwalletjournalentry",
            ),
        ),
        migrations.AddField(
            model_name="corporationpayments",
            name="owner",
            field=models.ForeignKey(
                blank=True,
                help_text="Owner of this payment",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_payments",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepayments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.alliancepaymentaccount",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepayments",
            name="entry_id",
            field=models.BigIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="allianceupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("tax_accounts", "Tax Accounts"),
                    ("payments", "Payments"),
                    ("deadlines", "Deadlines"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.corporationpaymentaccount",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="entry_id",
            field=models.BigIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("division_names", "Wallet Division Names"),
                    ("divisions", "Wallet Divisions"),
                    ("wallet", "Wallet Journal"),
                    ("members", "Members"),
                    ("tax_accounts", "Tax Accounts"),
                    ("payments", "Payments"),
                    ("deadlines", "Deadlines"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
    ]



# Django
from django.core.handlers.wsgi import WSGIRequest
from django.utils.translation import gettext_lazy as _

# AA TaxSystem
from taxsystem.models.alliance import AllianceOwner
from taxsystem.models.corporation import CorporationOwner


def get_manage_owner(
    request: WSGIRequest, owner_id: int
) -> tuple[CorporationOwner | AllianceOwner | None, bool]:
    """
    Check if the user has manage permissions for the owner.
    Args:
        request (WSGIRequest): The HTTP request object containing user information from Alliance Auth
        owner_id (int): The ID of the owner to retrieve
    Returns:
        tuple: A tuple containing the owner object (or None if not found) and a boolean indicating permission
    """
    perms = True
    try:
        owner = CorporationOwner.objects.get(eve_corporation__corporation_id=owner_id)
        visible = CorporationOwner.objects.manage_to(request.user)
        if owner not in visible:
            perms = False
    except CorporationOwner.DoesNotExist:
        try:
            owner = AllianceOwner.objects.get(eve_alliance__alliance_id=owner_id)
            visible = AllianceOwner.objects.manage_to(request.user)
            if owner not in visible:
                perms = False
        except AllianceOwner.DoesNotExist:
            return None, None
    return owner, perms


def get_owner(
    request: WSGIRequest, owner_id: int
) -> tuple[CorporationOwner | AllianceOwner | None, bool]:
    """
    Check if the user has visibility permissions for the owner.
    Args:
        request (WSGIRequest): The HTTP request object containing user information from Alliance Auth
        owner_id (int): The ID of the owner to retrieve
    Returns:
        tuple: A tuple containing the owner object (or None if not found) and a boolean indicating permission
    """
    perms = True
    try:
        owner = CorporationOwner.objects.get(eve_corporation__corporation_id=owner_id)
        visible = CorporationOwner.objects.visible_to(request.user)
        if owner not in visible:
            perms = False
    except CorporationOwner.DoesNotExist:
        try:
            owner = AllianceOwner.objects.get(eve_alliance__alliance_id=owner_id)
            visible = AllianceOwner.objects.visible_to(request.user)
            if owner not in visible:
                perms = False
        except AllianceOwner.DoesNotExist:
            return None, False
    return owner, perms


def get_character_permissions(request, character_id) -> bool:
    """
    Check if the user has permissions for the character.
    Args:
        request (WSGIRequest): The HTTP request object containing user information from Alliance Auth
        character_id (int): The ID of the character to check permissions for
    Returns:
        bool: True if the user has permissions for the character, False otherwise
    """
    perms = True

    # Get all character IDs the user owns
    char_ids = request.user.character_ownerships.all().values_list(
        "character__character_id", flat=True
    )
    if character_id not in char_ids:
        perms = False
    return perms



# Django
from django.db import models
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _


class UpdateSection(models.TextChoices):
    """
    Base Class for Update Sections.
    """

    @classmethod
    def get_sections(cls) -> list[str]:
        """Return list of section values."""
        return [choice.value for choice in cls]

    @property
    def method_name(self) -> str:
        """Return method name for this section."""
        return f"update_{self.value}"


class UpdateStatus(models.TextChoices):
    """Status for ESI data updates.
    Used to indicate the overall status of an Owner.
    """

    DISABLED = "disabled", _("Disabled")
    TOKEN_ERROR = "token_error", _("Token Error")
    ERROR = "error", _("Error")
    OK = "ok", _("OK")
    INCOMPLETE = "incomplete", _("Incomplete")
    IN_PROGRESS = "in_progress", _("In Progress")

    def bootstrap_icon(self) -> str:
        """Return bootstrap corresponding icon class."""
        update_map = {
            status: mark_safe(
                f"<span class='{self.bootstrap_text_style_class()}' data-bs-tooltip='aa-taxsystem' title='{self.description()}'>⬤</span>"
            )
            for status in [
                self.DISABLED,
                self.TOKEN_ERROR,
                self.ERROR,
                self.INCOMPLETE,
                self.IN_PROGRESS,
                self.OK,
            ]
        }
        return update_map.get(self, "")

    def bootstrap_text_style_class(self) -> str:
        """Return bootstrap corresponding bootstrap text style class."""
        update_map = {
            self.DISABLED: "text-muted",
            self.TOKEN_ERROR: "text-warning",
            self.INCOMPLETE: "text-warning",
            self.IN_PROGRESS: "text-info",
            self.ERROR: "text-danger",
            self.OK: "text-success",
        }
        return update_map.get(self, "")

    def description(self) -> str:
        """Return description for an enum object."""
        update_map = {
            self.DISABLED: _("Update is disabled"),
            self.TOKEN_ERROR: _("One section has a token error during update"),
            self.INCOMPLETE: _("One or more sections have not been updated"),
            self.IN_PROGRESS: _("Update is in progress"),
            self.ERROR: _("An error occurred during update"),
            self.OK: _("Updates completed successfully"),
        }
        return update_map.get(self, "")


class AccountStatus(models.TextChoices):
    """Status for Tax Accounts.
    This indicates the current status of a tax account.
    """

    ACTIVE = "active", _("Active")
    INACTIVE = "inactive", _("Inactive")
    DEACTIVATED = "deactivated", _("Deactivated")
    MISSING = "missing", _("Missing")

    def html(self, text=False) -> mark_safe:
        """Return the HTML for the status."""
        if text:
            return format_html(
                f"<span class='badge bg-{self.color()}' data-bs-tooltip='aa-taxsystem' title='{self.label}'>{self.label}</span>"
            )
        return format_html(
            f"<span class='btn btn-sm btn-square bg-{self.color()}' data-bs-tooltip='aa-taxsystem' title='{self.label}'>{self.icon()}</span>"
        )

    def color(self) -> str:
        """Return bootstrap corresponding icon class."""
        status_map = {
            self.ACTIVE: "success",
            self.INACTIVE: "warning",
            self.DEACTIVATED: "danger",
            self.MISSING: "info",
        }
        return status_map.get(self, "secondary")

    def icon(self) -> str:
        """Return description for an enum object."""
        status_map = {
            self.ACTIVE: "<i class='fas fa-check'></i>",
            self.INACTIVE: "<i class='fas fa-user-slash'></i>",
            self.DEACTIVATED: "<i class='fas fa-user-clock'></i>",
            self.MISSING: "<i class='fas fa-question'></i> ",
        }
        return status_map.get(self, "")


class PaymentRequestStatus(models.TextChoices):
    APPROVED = "approved", _("Approved")
    PENDING = "pending", _("Pending")
    REJECTED = "rejected", _("Rejected")
    NEEDS_APPROVAL = "needs_approval", _("Requires Auditor")

    def color(self) -> str:
        """Return bootstrap corresponding icon class."""
        status_map = {
            self.APPROVED: "success",
            self.PENDING: "warning",
            self.REJECTED: "danger",
            self.NEEDS_APPROVAL: "info",
        }
        return status_map.get(self, "secondary")

    def alert(self) -> str:
        """Return bootstrap corresponding badge class."""
        status_map = {
            self.APPROVED: "alert alert-success",
            self.PENDING: "alert alert-warning",
            self.REJECTED: "alert alert-danger",
            self.NEEDS_APPROVAL: "alert alert-info",
        }
        alert_html = f"<div class='text-center alert {status_map.get(self, 'alert alert-secondary')}'>{self.label}</div>"
        return alert_html


class PaymentActions(models.TextChoices):
    DEFAULT = "", ""
    STATUS_CHANGE = "Status Changed", _("Status Changed")
    PAYMENT_ADDED = "Payment Added", _("Payment Added")
    CUSTOM_PAYMENT = "Custom Payment Added", _("Custom Payment Added")
    REVISER_COMMENT = "Reviser Comment", _("Reviser Comment")


class PaymentStatus(models.TextChoices):
    """Status for Payments.
    This indicates whether a payment has been made or not.
    """

    PAID = "paid", _("Paid")
    UNPAID = "unpaid", _("Unpaid")

    def color(self) -> str:
        """Return bootstrap corresponding icon class."""
        paid_map = {
            self.PAID: "success",
            self.UNPAID: "danger",
        }
        return paid_map.get(self, "secondary")


class PaymentSystemText(models.TextChoices):
    """
    Text choices for system changes in payment history.
    """

    DEFAULT = "", ""
    ADDED = "Payment added to system", _("Payment added to system")
    AUTOMATIC = "Automated approved Payment", _("Automated approved Payment")
    REVISER = "Payment must be approved by an reviser", _(
        "Payment must be approved by an reviser"
    )


class CorporationUpdateSection(UpdateSection):
    """Sections for corporation updates."""

    DIVISION_NAMES = "division_names", _("Wallet Division Names")
    DIVISIONS = "divisions", _("Wallet Divisions")
    WALLET = "wallet", _("Wallet Journal")
    MEMBERS = "members", _("Members")
    TAX_ACCOUNTS = "tax_accounts", _("Tax Accounts")
    PAYMENTS = "payments", _("Payments")
    DEADLINES = "deadlines", _("Deadlines")


class AllianceUpdateSection(UpdateSection):
    """Sections for alliance updates."""

    TAX_ACCOUNTS = "tax_accounts", _("Tax Accounts")
    PAYMENTS = "payments", _("Payments")
    DEADLINES = "deadlines", _("Deadlines")


class AdminActions(models.TextChoices):
    DEFAULT = "", ""
    ADD = "Added", _("Added")
    CHANGE = "Changed", _("Changed")
    DELETE = "Deleted", _("Deleted")


class FilterMatchType(models.TextChoices):
    EXACT = "exact", _("Exact Match")
    CONTAINS = "contains", _("Contains")



"""
App Settings
"""

# Django
from django.conf import settings

# Set Naming on Auth Hook
TAXSYSTEM_APP_NAME = getattr(settings, "TAXSYSTEM_APP_NAME", "Tax System")

# Task Settings
# Global timeout for tasks in seconds to reduce task accumulation during outages.
TAXSYSTEM_TASKS_TIME_LIMIT = getattr(settings, "TAXSYSTEM_TASKS_TIME_LIMIT", 7200)

# Stale time in minutes for each type of data
TAXSYSTEM_STALE_TYPES = getattr(
    settings,
    "TAXSYSTEM_STALE_TYPES",
    {
        "wallet": 60,
        "division_names": 60,
        "divisions": 30,
        "members": 60,
        "payments": 60,
        "tax_accounts": 60,
        "deadlines": 1440,
    },
)

# Controls how many database records are inserted in a single batch operation.
TAXSYSTEM_BULK_BATCH_SIZE = getattr(settings, "TAXSYSTEM_BULK_BATCH_SIZE", 500)



# Standard Library
import json
from http import HTTPStatus

# Django
from django.contrib.humanize.templatetags.humanize import intcomma
from django.urls import reverse
from django.utils import timezone

# AA TaxSystem
from taxsystem.models.corporation import CorporationPaymentAccount
from taxsystem.models.general import EveEntity
from taxsystem.models.helpers.textchoices import PaymentActions, PaymentRequestStatus
from taxsystem.tests import TaxSystemTestCase
from taxsystem.tests.testdata.utils import (
    create_division,
    create_owner_from_user,
    create_payment,
    create_payment_history,
    create_tax_account,
    create_wallet_journal_entry,
)

MODULE_PATH = "taxsystem.api.helpers."
API_URL = "taxsystem:api"


class TestPaymentsApiEndpoints(TaxSystemTestCase):
    """Test Payments API Endpoints."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        cls.audit = create_owner_from_user(user=cls.user)

        cls.division = create_division(
            corporation=cls.audit, balance=0, division_id=1, name="Main Division"
        )

        cls.first_party = EveEntity.objects.get(id=1002)
        cls.second_party = EveEntity.objects.get(id=1001)

        cls.tax_account = create_tax_account(
            name=cls.user_character.character.character_name,
            owner=cls.audit,
            user=cls.user,
            status="active",
            deposit=0,
        )

    def test_get_payments_should_200_basic_access(self):
        """
        Test that a user with 'basic_access' can access API Endpoint 'get_payments'.

        Results:
        - Access is granted
        - Payment from owner is included in the response
        """
        # Test Data
        corporation_id = self.user_character.character.corporation_id

        self.journal_entry = create_wallet_journal_entry(
            division=self.division,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            ref_type="tax_payment",
            first_party=self.first_party,
            second_party=self.second_party,
            description="Test Description",
        )

        # Approved Payment
        payment = create_payment(
            name=self.user_character.character.character_name,
            account=self.tax_account,
            owner=self.audit,
            journal=self.journal_entry,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            request_status=PaymentRequestStatus.PENDING,
            reviser="",
        )

        url = reverse(f"{API_URL}:get_payments", kwargs={"owner_id": corporation_id})
        self.client.force_login(self.user)

        # Test Action
        response = self.client.get(url)

        # Expected Result
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertIn(str(payment.amount), str(response.json()))

    def test_get_my_payments_should_200_basic_access(self):
        """
        Test that a user with 'basic_access' can access API Endpoint 'get_my_payments'.

        Results:
        - Access is granted
        - Payment from user is included in the response
        """
        # Test Data
        corporation_id = self.user_character.character.corporation_id

        self.journal_entry = create_wallet_journal_entry(
            division=self.division,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            ref_type="tax_payment",
            first_party=self.first_party,
            second_party=self.second_party,
            description="Test Description",
        )

        # Pending Payment
        payment = create_payment(
            name=self.user_character.character.character_name,
            account=self.tax_account,
            owner=self.audit,
            journal=self.journal_entry,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            request_status=PaymentRequestStatus.PENDING,
            reviser="",
        )

        url = reverse(f"{API_URL}:get_my_payments", kwargs={"owner_id": corporation_id})
        self.client.force_login(self.user)

        # Test Action
        response = self.client.get(url)

        # Expected Result
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertIn(str(payment.amount), str(response.json()))

    def test_get_member_payments(self):
        """
        Test 'api:get_member_payments' endpoint.

        # Test Szenarios:
            1. Member payments are returned successfully for 'manage_own_corp' Permission.
            2. Permission Denied for users without access.
            3. Member payments are returned successfully for superuser.
        """
        # Test Data
        corporation_id = self.user_character.character.corporation_id

        self.journal_entry = create_wallet_journal_entry(
            division=self.division,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            ref_type="tax_payment",
            first_party=self.first_party,
            second_party=self.second_party,
            description="Test Description",
        )

        # Pending Payment
        payment = create_payment(
            name=self.user_character.character.character_name,
            account=self.tax_account,
            owner=self.audit,
            journal=self.journal_entry,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            request_status=PaymentRequestStatus.PENDING,
            reviser="",
        )

        url = reverse(
            f"{API_URL}:get_member_payments",
            kwargs={
                "owner_id": corporation_id,
                "character_id": self.user_character.character.character_id,
            },
        )
        self.client.force_login(self.manage_own_user)

        # Test Action
        response = self.client.get(url)

        # Expected Result
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertIn(str(payment.amount), str(response.json()))

        # Test Data for Permission Denied
        url = reverse(
            f"{API_URL}:get_member_payments",
            kwargs={
                "owner_id": corporation_id,
                "character_id": self.user_character.character.character_id,
            },
        )
        self.client.force_login(self.user)

        # Test Action
        response = self.client.get(url)

        # Expected Result
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)
        self.assertNotIn(str(payment.amount), str(response.json()))

        # Test Data for Superuser Access
        url = reverse(
            f"{API_URL}:get_member_payments",
            kwargs={
                "owner_id": corporation_id,
                "character_id": self.user_character.character.character_id,
            },
        )
        self.client.force_login(self.superuser)

        # Test Action
        response = self.client.get(url)

        # Expected Result
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertIn(str(payment.amount), str(response.json()))

    def test_get_payment_details(self):
        """
        Test 'api:get_payment_details' endpoint.

        # Test Szenarios:
            1. Payment details are returned successfully.
            2. Permission Denied for users without access.
        """
        # Test Data
        corporation_id = self.user_character.character.corporation_id

        self.journal_entry = create_wallet_journal_entry(
            division=self.division,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            ref_type="tax_payment",
            first_party=self.first_party,
            second_party=self.second_party,
            description="Test Description",
        )

        # Pending Payment
        payment = create_payment(
            name=self.user_character.character.character_name,
            account=self.tax_account,
            owner=self.audit,
            journal=self.journal_entry,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            request_status=PaymentRequestStatus.PENDING,
            reviser="",
        )
        create_payment_history(
            payment=payment,
            user=self.user,
            new_status=PaymentRequestStatus.PENDING,
            action=PaymentActions.PAYMENT_ADDED,
        )

        url = reverse(
            f"{API_URL}:get_payment_details",
            kwargs={"owner_id": corporation_id, "payment_pk": payment.pk},
        )
        self.client.force_login(self.manage_own_user)

        # Test Action
        response = self.client.get(url)

        # Expected Result
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertIn(str(payment.amount), str(response.json()))
        self.assertIn(payment.reason, str(response.json()))
        self.assertIn(str(self.audit.name), str(response.json()))

        # Test Data for Permission Denied
        url = reverse(
            f"{API_URL}:get_payment_details",
            kwargs={"owner_id": corporation_id, "payment_pk": payment.pk},
        )
        self.client.force_login(self.user2)

        # Test Action
        response = self.client.get(url)

        # Expected Result
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)
        self.assertNotIn(str(payment.amount), str(response.json()))
        self.assertNotIn(payment.reason, str(response.json()))
        self.assertNotIn(str(self.audit.name), str(response.json()))

    def test_add_payment(self):
        """
        Test 'api:add_payment' endpoint.

        # Test Szenarios:
            1. Payment is added successfully.
            2. Permission Denied for users without manage access.
        """
        # Test Data
        corporation_id = self.user_character.character.corporation_id

        self.journal_entry = create_wallet_journal_entry(
            division=self.division,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            ref_type="tax_payment",
            first_party=self.first_party,
            second_party=self.second_party,
            description="Test Description",
        )

        url = reverse(
            f"{API_URL}:add_payment",
            kwargs={"owner_id": corporation_id, "account_pk": self.tax_account.pk},
        )
        self.client.force_login(self.manage_own_user)

        data = {
            "amount": 1000,
            "comment": "Adding payment via API test.",
        }

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = "Custom Payment Added: {reason}".format(reason=data["comment"])
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertEqual(response.json().get("message"), result)

        # Test Data for Permission Denied
        url = reverse(
            f"{API_URL}:add_payment",
            kwargs={"owner_id": corporation_id, "account_pk": self.tax_account.pk},
        )
        self.client.force_login(self.user)

        data = {
            "amount": 1000,
            "comment": "Adding payment via API test.",
        }

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = "Permission Denied."
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)
        self.assertEqual(response.json().get("error"), result)

    def test_approve_payment(self):
        """
        Test approve payment endpoint.

        # Test Szenarios:
            1. Payment is approved successfully.
            2. Permission Denied for users without manage access.
        """
        # Test Data
        corporation_id = self.user_character.character.corporation_id

        self.journal_entry = create_wallet_journal_entry(
            division=self.division,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            ref_type="tax_payment",
            first_party=self.first_party,
            second_party=self.second_party,
            description="Test Description",
        )

        # Pending Payment
        payment = create_payment(
            name=self.user_character.character.character_name,
            account=self.tax_account,
            owner=self.audit,
            journal=self.journal_entry,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            request_status=PaymentRequestStatus.PENDING,
            reviser="",
        )

        url = reverse(
            f"{API_URL}:approve_payment",
            kwargs={"owner_id": corporation_id, "payment_pk": payment.pk},
        )
        self.client.force_login(self.manage_own_user)

        data = {
            "comment": "Approving payment via API test.",
        }

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = "Payment ID: {pid} - Amount: {amount} - Name: {name} approved".format(
            pid=payment.pk, amount=intcomma(payment.amount), name=payment.name
        )
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertEqual(response.json().get("message"), result)

        # Test Data for Permission Denied
        url = reverse(
            f"{API_URL}:approve_payment",
            kwargs={"owner_id": corporation_id, "payment_pk": payment.pk},
        )
        self.client.force_login(self.user)

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = "Permission Denied."
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)
        self.assertEqual(response.json().get("error"), result)

    def test_undo_payment(self):
        """
        Test undo payment endpoint.

        # Test Szenarios:
            1. Rejected Payment is undone successfully (should not change deposit).
            2. Approved Payment is undone successfully (should change deposit).
            3. Permission Denied for users without manage access.
        """
        # Test Data
        corporation_id = self.user_character.character.corporation_id

        self.journal_entry = create_wallet_journal_entry(
            division=self.division,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            ref_type="tax_payment",
            first_party=self.first_party,
            second_party=self.second_party,
            description="Test Description",
        )

        self.journal_entry2 = create_wallet_journal_entry(
            division=self.division,
            entry_id=2,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            ref_type="tax_payment",
            first_party=self.first_party,
            second_party=self.second_party,
            description="Test Description",
        )

        reject_payment = create_payment(
            name=self.user_character.character.character_name,
            account=self.tax_account,
            owner=self.audit,
            journal=self.journal_entry2,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            request_status=PaymentRequestStatus.REJECTED,
            reviser="",
        )

        url = reverse(
            f"{API_URL}:undo_payment",
            kwargs={"owner_id": corporation_id, "payment_pk": reject_payment.pk},
        )
        self.client.force_login(self.manage_own_user)

        data = {
            "comment": "Undoing payment via API test.",
        }

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = "Payment ID: {pid} - Amount: {amount} - Name: {name} undone".format(
            pid=reject_payment.pk,
            amount=intcomma(reject_payment.amount),
            name=reject_payment.name,
        )
        tax_account = CorporationPaymentAccount.objects.get(pk=self.tax_account.pk)
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertEqual(response.json().get("message"), result)
        self.assertEqual(tax_account.deposit, 0)

        # Test Data for Approved Payment
        approved_payment = create_payment(
            name=self.user_character.character.character_name,
            account=self.tax_account,
            owner=self.audit,
            journal=self.journal_entry,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            request_status=PaymentRequestStatus.APPROVED,
            reviser="",
        )

        url = reverse(
            f"{API_URL}:undo_payment",
            kwargs={"owner_id": corporation_id, "payment_pk": approved_payment.pk},
        )
        self.client.force_login(self.manage_own_user)

        data = {
            "comment": "Undoing payment via API test.",
        }

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = "Payment ID: {pid} - Amount: {amount} - Name: {name} undone".format(
            pid=approved_payment.pk,
            amount=intcomma(approved_payment.amount),
            name=approved_payment.name,
        )
        tax_account = CorporationPaymentAccount.objects.get(pk=self.tax_account.pk)
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertEqual(response.json().get("message"), result)
        self.assertEqual(tax_account.deposit, -1000)

        # Test Data for Permission Denied
        url = reverse(
            f"{API_URL}:undo_payment",
            kwargs={"owner_id": corporation_id, "payment_pk": approved_payment.pk},
        )
        self.client.force_login(self.user)

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = "Permission Denied."
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)
        self.assertEqual(response.json().get("error"), result)

    def test_delete_payment(self):
        """
        Test delete payment endpoint.

        # Test Szenarios:
            1. Custom Payment is deleted successfully.
            2. Permission Denied for users without manage access.
            3. ESI imported Payments cannot be deleted.
        """
        # Test Data
        corporation_id = self.user_character.character.corporation_id

        self.journal_entry = create_wallet_journal_entry(
            division=self.division,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            ref_type="tax_payment",
            first_party=self.first_party,
            second_party=self.second_party,
            description="Test Description",
        )

        pending_payment = create_payment(
            name="Pending Payment",
            account=self.tax_account,
            owner=self.audit,
            journal=self.journal_entry,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            request_status=PaymentRequestStatus.PENDING,
            reviser="",
        )

        # Pending Payment
        custom_payment = create_payment(
            name="Custom Payment",
            account=self.tax_account,
            owner=self.audit,
            journal=None,
            amount=2000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            request_status=PaymentRequestStatus.PENDING,
            reviser="",
        )

        url = reverse(
            f"{API_URL}:delete_payment",
            kwargs={"owner_id": corporation_id, "payment_pk": custom_payment.pk},
        )
        self.client.force_login(self.manage_own_user)

        data = {
            "comment": "Deleting payment via API test.",
        }

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = "Payment ID: {pid} - Amount: {amount} - Name: {name} deleted - {reason}".format(
            pid=custom_payment.pk,
            amount=intcomma(custom_payment.amount),
            name=custom_payment.name,
            reason=data["comment"],
        )
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertEqual(response.json().get("message"), result)

        # Test Data for Permission Denied
        url = reverse(
            f"{API_URL}:delete_payment",
            kwargs={"owner_id": corporation_id, "payment_pk": pending_payment.pk},
        )
        self.client.force_login(self.user)

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = "Permission Denied."
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)
        self.assertEqual(response.json().get("error"), result)

        # Test Data for Payments that cannot be deleted
        url = reverse(
            f"{API_URL}:delete_payment",
            kwargs={"owner_id": corporation_id, "payment_pk": pending_payment.pk},
        )
        self.client.force_login(self.manage_own_user)

        data = {
            "comment": "Deleting payment via API test.",
        }

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = "ESI imported payments cannot be deleted"
        self.assertEqual(response.status_code, HTTPStatus.BAD_REQUEST)
        self.assertEqual(response.json().get("message"), result)

    def test_reject_payment(self):
        """
        Test reject payment endpoint.

        # Test Szenarios:
            1. Payment is rejected successfully.
            2. Permission Denied for users without manage access.
        """
        # Test Data
        corporation_id = self.user_character.character.corporation_id

        self.journal_entry = create_wallet_journal_entry(
            division=self.division,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            ref_type="tax_payment",
            first_party=self.first_party,
            second_party=self.second_party,
            description="Test Description",
        )

        pending_payment = create_payment(
            name="Pending Payment",
            account=self.tax_account,
            owner=self.audit,
            journal=self.journal_entry,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            request_status=PaymentRequestStatus.PENDING,
            reviser="",
        )

        url = reverse(
            f"{API_URL}:reject_payment",
            kwargs={"owner_id": corporation_id, "payment_pk": pending_payment.pk},
        )
        self.client.force_login(self.manage_own_user)

        data = {
            "comment": "Rejecting payment via API test.",
        }

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = "Payment ID: {pid} - Amount: {amount} - Name: {name} rejected - {reason}".format(
            pid=pending_payment.pk,
            amount=intcomma(pending_payment.amount),
            name=pending_payment.name,
            reason=data["comment"],
        )
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertEqual(response.json().get("message"), result)

        # Test Data for Permission Denied
        url = reverse(
            f"{API_URL}:reject_payment",
            kwargs={"owner_id": corporation_id, "payment_pk": pending_payment.pk},
        )
        self.client.force_login(self.user)

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = "Permission Denied."
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)
        self.assertEqual(response.json().get("error"), result)

    def test_bulk_actions(self):
        """
        Test bulk actions endpoint.

        # Test Szenarios:
            1. Payments are approved successfully in bulk.
            2. Payments are rejected successfully in bulk.
            3. Permission Denied for users without manage access.
        """
        # Test Data
        corporation_id = self.user_character.character.corporation_id

        # Pending Payment 1
        payment1 = create_payment(
            name="Pending Payment 1",
            account=self.tax_account,
            owner=self.audit,
            journal=None,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            request_status=PaymentRequestStatus.PENDING,
            reviser="",
        )

        # Pending Payment 2
        payment2 = create_payment(
            name="Pending Payment 2",
            account=self.tax_account,
            owner=self.audit,
            journal=None,
            amount=2000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0),
            reason="Tax Payment",
            request_status=PaymentRequestStatus.PENDING,
            reviser="",
        )

        url = reverse(
            f"{API_URL}:perform_bulk_actions_payments",
            kwargs={"owner_id": corporation_id},
        )
        self.client.force_login(self.manage_own_user)

        data = {
            "pks": [payment1.pk, payment2.pk],
            "action": "approve",
        }

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result (match API: list-format pks and no trailing period)
        result = (
            "Bulk '{status}' performed for {runs} payments ({pks}) for {owner}".format(
                status=PaymentRequestStatus.APPROVED,
                runs=len(data["pks"]),
                pks=str(data["pks"]),
                owner=self.audit.name,
            )
        )
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertEqual(response.json().get("message"), result)

        # Test Data for Reject Action
        payment1.request_status = PaymentRequestStatus.PENDING
        payment1.save()
        payment2.request_status = PaymentRequestStatus.PENDING
        payment2.save()

        url = reverse(
            f"{API_URL}:perform_bulk_actions_payments",
            kwargs={"owner_id": corporation_id},
        )
        self.client.force_login(self.manage_own_user)
        data = {
            "pks": [payment1.pk, payment2.pk],
            "action": "reject",
        }

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )
        # Expected Result (match API: list-format pks and no trailing period)
        result = (
            "Bulk '{status}' performed for {runs} payments ({pks}) for {owner}".format(
                status=PaymentRequestStatus.REJECTED,
                runs=len(data["pks"]),
                pks=str(data["pks"]),
                owner=self.audit.name,
            )
        )
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertEqual(response.json().get("message"), result)

        # Test Data for Permission Denied
        url = reverse(
            f"{API_URL}:perform_bulk_actions_payments",
            kwargs={"owner_id": corporation_id},
        )
        self.client.force_login(self.user)
        data = {
            "pks": [payment1.pk, payment2.pk],
            "action": "approve",
        }
        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )
        # Expected Result
        result = "Permission Denied."
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)
        self.assertEqual(response.json().get("error"), result)



# Third Party
from ninja import Schema

# Django
from django.db.models import Count, F, Q
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# AA TaxSystem
from taxsystem import __title__
from taxsystem.api.schema import (
    DashboardDivisionsSchema,
    DivisionSchema,
    UpdateStatusSchema,
)
from taxsystem.models.alliance import (
    AllianceOwner,
)
from taxsystem.models.corporation import (
    CorporationOwner,
    Members,
)
from taxsystem.models.helpers.textchoices import AccountStatus, PaymentRequestStatus
from taxsystem.providers import AppLogger

logger = AppLogger(get_extension_logger(__name__), __title__)


class TaxAccountStatisticsSchema(Schema):
    accounts: int
    accounts_active: int
    accounts_inactive: int
    accounts_deactivated: int
    accounts_paid: int
    accounts_unpaid: int


class PaymentsStatisticsSchema(Schema):
    payments: int
    payments_pending: int
    payments_automatic: int
    payments_manual: int


class MembersStatisticsSchema(Schema):
    members: int
    members_unregistered: int
    members_alts: int
    members_mains: int


class StatisticsResponse(Schema):
    owner_id: int | None = None
    owner_name: str | None = None
    tax_account: TaxAccountStatisticsSchema
    payments: PaymentsStatisticsSchema
    members: MembersStatisticsSchema


def create_dashboard_common_data(owner, divisions):
    """
    Create common dashboard data structure

    Args:
        owner: Owner object (CorporationOwner or AllianceOwner)
        divisions: QuerySet of CorporationWalletDivision objects

    Returns:
        dict: Dictionary containing common dashboard data
    """
    # Create divisions
    response_divisions_list = []
    total_balance = 0

    # Get divisions and calculate total balance
    for i, division in enumerate(divisions, start=1):
        division_name = division.name if division.name else f"{i}. {_('Division')}"
        response_divisions_list.append(
            DivisionSchema(
                name=division_name,
                balance=division.balance,
            )
        )
        total_balance += division.balance

    # Create statistics
    response_statistics = StatisticsResponse(
        owner_id=owner.pk,
        owner_name=owner.name,
        tax_account=get_tax_account_statistics(owner),
        payments=get_payments_statistics(owner),
        members=get_members_statistics(owner),
    )

    return {
        "update_status": UpdateStatusSchema(
            status=owner.get_update_status,
            icon=owner.get_status.bootstrap_icon(),
        ),
        "tax_amount": owner.tax_amount,
        "tax_period": owner.tax_period,
        "divisions": DashboardDivisionsSchema(
            divisions=response_divisions_list,
            total_balance=total_balance,
        ),
        "statistics": response_statistics,
    }


def get_payments_statistics(
    owner: CorporationOwner | AllianceOwner,
) -> PaymentsStatisticsSchema:
    """Get payments statistics for an Owner."""
    payments = owner.payment_model.objects.filter(owner=owner)

    payments_statistics = payments.aggregate(
        total=Count("id"),
        automatic=Count("id", filter=Q(reviser="System")),
        manual=Count("id", filter=~Q(reviser="System") & ~Q(reviser="")),
        pending=Count(
            "id",
            filter=Q(
                request_status__in=[
                    PaymentRequestStatus.PENDING,
                    PaymentRequestStatus.NEEDS_APPROVAL,
                ]
            ),
        ),
    )

    return PaymentsStatisticsSchema(
        payments=payments_statistics["total"],
        payments_pending=payments_statistics["pending"],
        payments_automatic=payments_statistics["automatic"],
        payments_manual=payments_statistics["manual"],
    )


def get_tax_account_statistics(
    owner: CorporationOwner | AllianceOwner,
) -> TaxAccountStatisticsSchema:
    """Get tax account statistics for an Owner."""
    tax_accounts = owner.account_model.objects.filter(owner=owner)
    period = timezone.timedelta(days=owner.tax_period)

    statistics = tax_accounts.exclude(status=AccountStatus.MISSING).aggregate(
        users=Count("id"),
        active=Count("id", filter=Q(status=AccountStatus.ACTIVE)),
        inactive=Count("id", filter=Q(status=AccountStatus.INACTIVE)),
        deactivated=Count("id", filter=Q(status=AccountStatus.DEACTIVATED)),
        paid=Count(
            "id",
            filter=(
                Q(deposit__gte=F("owner__tax_amount"))
                | (
                    Q(last_paid__isnull=False)
                    & Q(deposit__gte=0)
                    & Q(last_paid__gte=timezone.now() - period)
                )
            )
            & Q(status=AccountStatus.ACTIVE),
        ),
    )
    # Calculate unpaid count
    unpaid = statistics["active"] - statistics["paid"]

    return TaxAccountStatisticsSchema(
        accounts=statistics["users"],
        accounts_active=statistics["active"],
        accounts_inactive=statistics["inactive"],
        accounts_deactivated=statistics["deactivated"],
        accounts_paid=statistics["paid"],
        accounts_unpaid=unpaid,
    )


def get_members_statistics(
    owner: CorporationOwner | AllianceOwner,
) -> MembersStatisticsSchema:
    # Determine the correct filter based on alliance system setting
    if isinstance(owner, CorporationOwner):
        # Return all members in the corporation
        members = Members.objects.filter(owner=owner).order_by("character_name")
    else:
        # Return all members in the alliance
        members = Members.objects.filter(
            owner__eve_corporation__alliance=owner.eve_alliance
        ).order_by("character_name")

    members_statistics = members.aggregate(
        total=Count("character_id"),
        unregistered=Count("character_id", filter=Q(status=Members.States.NOACCOUNT)),
        alts=Count("character_id", filter=Q(status=Members.States.IS_ALT)),
        mains=Count("character_id", filter=Q(status=Members.States.ACTIVE)),
    )

    return MembersStatisticsSchema(
        members=members_statistics["total"],
        members_unregistered=members_statistics["unregistered"],
        members_alts=members_statistics["alts"],
        members_mains=members_statistics["mains"],
    )



"""Hook into Alliance Auth"""

# Django
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth import hooks
from allianceauth.authentication.models import UserProfile
from allianceauth.services.hooks import MenuItemHook, UrlHook, get_extension_logger

# AA TaxSystem
from taxsystem import __title__, app_settings, urls
from taxsystem.models.corporation import (
    CorporationPaymentAccount,
    CorporationPayments,
)
from taxsystem.providers import AppLogger

logger = AppLogger(get_extension_logger(__name__), __title__)


class TaxSystemMenuItem(MenuItemHook):
    """This class ensures only authorized users will see the menu entry"""

    def __init__(self):
        super().__init__(
            f"{app_settings.TAXSYSTEM_APP_NAME}",
            "fas fa-landmark fa-fw",
            "taxsystem:index",
            navactive=["taxsystem:"],
        )

    def render(self, request: UserProfile):
        if request.user.has_perm("taxsystem.basic_access"):
            # Check if the User has Paid for the current period and set count to 1 if not paid, otherwise 0
            try:
                payment_user = CorporationPaymentAccount.objects.get(user=request.user)
                self.count = 1 if not payment_user.has_paid else 0
            except CorporationPaymentAccount.DoesNotExist:
                self.count = 0

            if request.user.has_perm(
                "taxsystem.manage_own_corp"
            ) or request.user.has_perm("taxsystem.manage_corps"):
                # Get the count of open invoices for the Managing user
                invoices = CorporationPayments.objects.get_visible_open_invoices(
                    request.user
                )
                self.count = invoices if invoices and invoices > 0 else self.count
            return MenuItemHook.render(self, request)
        return ""


@hooks.register("menu_item_hook")
def register_menu():
    """Register the menu item"""

    return TaxSystemMenuItem()


@hooks.register("url_hook")
def register_urls():
    """Register app urls"""

    return UrlHook(urls, "taxsystem", r"^taxsystem/")



# Generated by Django 4.2.26 on 2025-11-14 13:34

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0010_create_alliance_corporation"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="corporationowner",
            options={"default_permissions": ()},
        ),
        migrations.AlterModelOptions(
            name="corporationpaymentaccount",
            options={"default_permissions": ()},
        ),
        migrations.AlterModelOptions(
            name="corporationpayments",
            options={"default_permissions": ()},
        ),
        migrations.RenameField(
            model_name="corporationowner",
            old_name="corporation",
            new_name="eve_corporation",
        ),
        migrations.RemoveField(
            model_name="corporationowner",
            name="alliance",
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corp_payments",
                to="taxsystem.corporationpaymentaccount",
            ),
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_update_status",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="journalfilter",
            name="filter_set",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_filters",
                to="taxsystem.journalfilterset",
            ),
        ),
        migrations.AlterField(
            model_name="journalfilterset",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_filter_set",
                to="taxsystem.corporationowner",
            ),
        ),
    ]



# Generated by Django 4.2.11 on 2025-02-16 15:37

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="paymenthistory",
            name="comment",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Payment added to system", "Payment added to system"),
                    ("Automated approved Payment", "Automated approved Payment"),
                    (
                        "Payment must be approved by an reviser",
                        "Payment must be approved by an reviser",
                    ),
                ],
                default="",
                help_text="Comment of the action",
                max_length=128,
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="paymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=16,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="payments",
            name="request_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                default="pending",
                max_length=16,
                verbose_name="Request Status",
            ),
        ),
    ]



# Standard Library
import json
from http import HTTPStatus

# Django
from django.urls import reverse

# AA TaxSystem
from taxsystem.models.corporation import CorporationFilter, CorporationFilterSet
from taxsystem.tests import TaxSystemTestCase
from taxsystem.tests.testdata.utils import (
    create_filter,
    create_filterset,
    create_owner_from_user,
)

MODULE_PATH = "taxsystem.api.helpers."
API_URL = "taxsystem:api"


class TestFilterApiEndpoints(TaxSystemTestCase):
    """Test Filter API Endpoints."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        cls.audit = create_owner_from_user(user=cls.user)

        cls.filterset = create_filterset(owner=cls.audit, name="Test FilterSet")

    def test_get_filters(self):
        """
        Test 'api:get_filters' Endpoint.

        # Test Scenarios:
            1. Filters are returned successfully.
            2. Permission Denied for users without access.
        """
        # Test Data
        create_filter(
            filter_set=self.filterset,
            filter_type=CorporationFilter.FilterType.AMOUNT,
            value=1000,
        )

        url = reverse(
            f"{API_URL}:get_filters",
            kwargs={"owner_id": self.audit.eve_id, "filterset_pk": self.filterset.pk},
        )
        self.client.force_login(self.superuser)

        # Test Action
        response = self.client.get(url)

        # Expected Result
        data = json.loads(response.content)
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertEqual(data[0]["value"]["raw"], "1000")
        self.assertEqual(data[0]["filter_type"], "Amount")

        # Test Scenario 2: Permission Denied
        url = reverse(
            f"{API_URL}:get_filters",
            kwargs={"owner_id": self.audit.eve_id, "filterset_pk": self.filterset.pk},
        )
        self.client.force_login(self.user)

        # Test Action
        response = self.client.get(url)

        # Expected Result
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)

    def test_delete_filter(self):
        """
        Test 'api:delete_filter' Endpoint.

        # Test Scenarios:
            1. Filter is deleted successfully.
            2. Permission Denied for users without access.
        """
        # Test Data
        filter_instance = create_filter(
            filter_set=self.filterset,
            filter_type=CorporationFilter.FilterType.AMOUNT,
            value=500,
        )

        url = reverse(
            f"{API_URL}:delete_filter",
            kwargs={"owner_id": self.audit.eve_id, "filter_pk": filter_instance.pk},
        )
        self.client.force_login(self.superuser)
        data = {"comment": "Delete filter via API test."}

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = '{filter_obj} in "{filter_set}" deleted - Reason: {reason}'.format(
            filter_obj=filter_instance,
            filter_set=self.filterset.name,
            reason=data["comment"],
        )
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertEqual(response.json().get("message"), result)

        # Test Scenario 2: Permission Denied
        url = reverse(
            f"{API_URL}:delete_filter",
            kwargs={"owner_id": self.audit.eve_id, "filter_pk": filter_instance.pk},
        )
        self.client.force_login(self.user)
        data = {"comment": "Delete filter via API test."}

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = "Permission Denied."
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)
        self.assertEqual(response.json().get("error"), result)

    def test_get_filter_set(self):
        """
        Test 'api:get_filter_set' Endpoint.

        # Test Scenarios:
            1. Filter Sets are returned successfully.
            2. Permission Denied for users without access.
        """
        # Test Data
        url = reverse(
            f"{API_URL}:get_filter_set", kwargs={"owner_id": self.audit.eve_id}
        )
        self.client.force_login(self.superuser)

        # Test Action
        response = self.client.get(url)

        # Expected Result
        data = json.loads(response.content)
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertEqual(data[0]["name"], self.filterset.name)

        # Test Scenario 2: Permission Denied
        url = reverse(
            f"{API_URL}:get_filter_set", kwargs={"owner_id": self.audit.eve_id}
        )
        self.client.force_login(self.user)

        # Test Action
        response = self.client.get(url)

        # Expected Result
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)

    def test_delete_filter_set(self):
        """
        Test 'api:delete_filter_set' Endpoint.

        # Test Scenarios:
            1. Filter Set is deleted successfully.
            2. Permission Denied for users without access.
        """
        # Test Data
        url = reverse(
            f"{API_URL}:delete_filter_set",
            kwargs={"owner_id": self.audit.eve_id, "filterset_pk": self.filterset.pk},
        )
        self.client.force_login(self.superuser)
        data = {"comment": "Delete filter set via API test."}

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = "{filter_set} deleted - Reason: {reason}".format(
            filter_set=self.filterset, reason=data["comment"]
        )
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertEqual(response.json().get("message"), result)

        # Test Scenario 2: Permission Denied
        url = reverse(
            f"{API_URL}:delete_filter_set",
            kwargs={"owner_id": self.audit.eve_id, "filterset_pk": self.filterset.pk},
        )
        self.client.force_login(self.user)
        data = {"comment": "Delete filter set via API test."}

        # Test Action
        response = self.client.post(
            path=url, data=json.dumps(data), content_type="application/json"
        )

        # Expected Result
        result = "Permission Denied."
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)
        self.assertEqual(response.json().get("error"), result)

    def test_switch_filter_set(self):
        """
        Test 'api:switch_filter_set' Endpoint.

        # Test Scenarios:
            1. Filter Set is switched successfully.
            2. Permission Denied for users without access.
        """
        # Test Data
        url = reverse(
            f"{API_URL}:switch_filter_set",
            kwargs={"owner_id": self.audit.eve_id, "filterset_pk": self.filterset.pk},
        )
        self.client.force_login(self.superuser)

        # Test Action
        response = self.client.post(path=url)

        # Expected Result
        filter_set = CorporationFilterSet.objects.get(pk=self.filterset.pk)
        result = "{filter_set} switched to {enabled}".format(
            filter_set=filter_set, enabled=filter_set.enabled
        )
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertEqual(response.json().get("message"), result)

        # Test Scenario 2: Permission Denied
        url = reverse(
            f"{API_URL}:switch_filter_set",
            kwargs={"owner_id": self.audit.eve_id, "filterset_pk": self.filterset.pk},
        )
        self.client.force_login(self.user)

        # Test Action
        response = self.client.post(path=url)

        # Expected Result
        result = "Permission Denied."
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)
        self.assertEqual(response.json().get("error"), result)
